/*! \file DynBattleDisplay.cpp

    \brief DynRPG plugin to display health, mana, and ATB gauges, plus status condition icons and counters, for Battlers

    \author Bernard J. Badger, AKA Aubrey the Bard

    DynBattleDisplay is a DynRPG plugin which allows the display of gauges above both heroes and
    monsters, plus icons and counters for status conditions. It allows for control over which parts
    of the display are shown over heroes or monsters, when they are shown, and details of how they
    are shown.
*/

//#define DYNBATTLEDISPLAY_DEBUG              // Comment out to remove debugging

#include <DynRPG/DynRPG.h>
#include <BattleOverlay.h>
#include <map>
#include <vector>
#ifdef DYNBATTLEDISPLAY_DEBUG
#include <sstream>          // Only needed for debug purposes
#include <iostream>         // Only needed for debug purposes
#include <fstream>          // Only needed for debug purposes
#endif // DYNBATTLEDISPLAY_DEBUG

// Use this as a template for debug popups
//// DEBUG
//std::stringstream message;
//message << "message";
//MessageBox(
//            NULL, // We don't need a window handle
//            message.str().c_str(), // Text
//            "Debug", // Title
//            MB_OK // Flags
//);
//// /DEBUG

// Use this as a template for writing to the debug file
//// DEBUG
//*ofDebug << "message\n";
//// /DEBUG

//! Battle display for a single Battler
/*!
    This class handles the modeling and displaying of battle information for a single Battler (hero
    or monster) including gauges for health, mana, and ATB, plus icons and counters for status
    conditions.
*/
class BattleDisplay
{

public:

    const static int GAUGE_WIDTH = 48;                  //!< Width of a gauge Image
    const static int GAUGE_HEIGHT = 16;                 //!< Height of a gauge Image
    const static int HEALTH_GAUGE_SRC_X = 0;            //!< Source X coordinate of health gauge in SystemGraphic
    const static int HEALTH_GAUGE_SRC_Y = 32;           //!< Source Y coordinate of health gauge in SystemGraphic
    const static int MANA_GAUGE_SRC_X = 0;              //!< Source X coordinate of mana gauge in SystemGraphic
    const static int MANA_GAUGE_SRC_Y = 48;             //!< Source Y coordinate of mana gauge in SystemGraphic
    const static int ATB_GAUGE_SRC_X = 0;               //!< Source X coordinate of ATB gauge in SystemGraphic
    const static int ATB_GAUGE_SRC_Y = 64;              //!< Source Y coordinate of ATB gauge in SystemGraphic
    const static int BAR_WIDTH = 16;                    //!< Width of a bar Image
    const static int BAR_HEIGHT = 16;                   //!< Height of a bar Image
    const static int HEALTH_BAR_A_SRC_X = 48;           //!< Source X coordinate of health bar A in SystemGraphic
    const static int HEALTH_BAR_A_SRC_Y = 32;           //!< Source Y coordinate of health bar A in SystemGraphic
    const static int MANA_BAR_A_SRC_X = 48;             //!< Source X coordinate of mana bar A in SystemGraphic
    const static int MANA_BAR_A_SRC_Y = 48;             //!< Source Y coordinate of mana bar A in SystemGraphic
    const static int ATB_BAR_A_SRC_X = 48;              //!< Source X coordinate of ATB bar A in SystemGraphic
    const static int ATB_BAR_A_SRC_Y = 64;              //!< Source Y coordinate of ATB bar A in SystemGraphic
    const static int HEALTH_BAR_B_SRC_X = 64;           //!< Source X coordinate of health bar B in SystemGraphic
    const static int HEALTH_BAR_B_SRC_Y = 32;           //!< Source Y coordinate of health bar B in SystemGraphic
    const static int MANA_BAR_B_SRC_X = 64;             //!< Source X coordinate of mana bar B in SystemGraphic
    const static int MANA_BAR_B_SRC_Y = 48;             //!< Source Y coordinate of mana bar B in SystemGraphic
    const static int ATB_BAR_B_SRC_X = 64;              //!< Source X coordinate of ATB bar B in SystemGraphic
    const static int ATB_BAR_B_SRC_Y = 64;              //!< Source Y coordinate of ATB bar B in SystemGraphic
    const static int FAST_ATB_BAR_A_SRC_X = 48;         //!< Source X coordinate of fast ATB bar A in SystemGraphic
    const static int FAST_ATB_BAR_A_SRC_Y = 0;          //!< Source Y coordinate of fast ATB bar A in SystemGraphic
    const static int FAST_ATB_BAR_B_SRC_X = 64;         //!< Source X coordinate of fast ATB bar B in SystemGraphic
    const static int FAST_ATB_BAR_B_SRC_Y = 0;          //!< Source Y coordinate of fast ATB bar B in SystemGraphic
    const static int SLOW_ATB_BAR_A_SRC_X = 48;         //!< Source X coordinate of slow ATB bar A in SystemGraphic
    const static int SLOW_ATB_BAR_A_SRC_Y = 16;         //!< Source Y coordinate of slow ATB bar A in SystemGraphic
    const static int SLOW_ATB_BAR_B_SRC_X = 64;         //!< Source X coordinate of slow ATB bar B in SystemGraphic
    const static int SLOW_ATB_BAR_B_SRC_Y = 16;         //!< Source Y coordinate of slow ATB bar B in SystemGraphic
    const static int DIGIT_WIDTH = 8;                   //!< Width of a digit Image
    const static int DIGIT_HEIGHT = 16;                 //!< Height of a digit Image
    const static int DIGIT_SRC_X = 0;                   //!< Source X coordinate of the first digit in SystemGraphic
    const static int DIGIT_SRC_Y = 80;                  //!< Source Y coordinate of the first digit in SystemGraphic
    const static int NUM_DIGITS = 10;                   //!< Amount of digit Images

    const static int HERO_SPRITE_WIDTH = 48;            //!< Default value for sHeroSpriteWidth
    const static int HERO_SPRITE_HEIGHT = 48;           //!< Default value for sHeroSpriteHeight
    const static int CONDITION_ICON_WIDTH = 12;         //!< Default value for sConditionIconWidth
    const static int CONDITION_ICON_HEIGHT = 12;        //!< Default value for sConditionIconHeight

    const static int HERO_HEALTH_GAUGE_DURATION = -1;   //!< Default value for sHeroHealthGaugeDuration
    const static int MONSTER_HEALTH_GAUGE_DURATION = -1;    //!< Default value for sMonsterHealthGaugeDuration
    const static int HERO_MANA_GAUGE_DURATION = -1;     //!< Default value for sHeroManaGaugeDuration
    const static int MONSTER_MANA_GAUGE_DURATION = -1;  //!< Default value for sMonsterManaGaugeDuration
    const static int HERO_HEALTH_CHANGE_PAUSE = 30;     //!< Default value for sHeroHealthChangePause
    const static int MONSTER_HEALTH_CHANGE_PAUSE = 30;  //!< Default value for sMonsterHealthChangePause
    const static int HERO_MANA_CHANGE_PAUSE = 30;       //!< Default value for sHeroManaChangePause
    const static int MONSTER_MANA_CHANGE_PAUSE = 30;    //!< Default value for sMonsterManaChangePause
    const static int HERO_HEALTH_CHANGE_DURATION = 30;  //!< Default value for sHeroHealthChangeDuration
    const static int MONSTER_HEALTH_CHANGE_DURATION = 30;   //!< Default value for sMonsterHealthChangeDuration
    const static int HERO_MANA_CHANGE_DURATION = 30;    //!< Default value for sHeroManaChangeDuration
    const static int MONSTER_MANA_CHANGE_DURATION = 30; //!< Default value for sMonsterManaChangeDuration

    const static int HERO_MAX_CONDITION_DISPLAY = 3;    //!< Default value for sHeroMaxConditionDisplay
    const static int MONSTER_MAX_CONDITION_DISPLAY = 3; //!< Default value for sMonsterMaxConditionDisplay
    const static int HERO_CONDITION_DURATION = 30;      //!< Default value for sHeroConditionDuration
    const static int MONSTER_CONDITION_DURATION = 30;   //!< Default value for sMonsterConditionDuration
    const static int HERO_CONDITION_BLINK_DURATION = 60;    //!< Default value for sHeroConditionBlinkDuration
    const static int MONSTER_CONDITION_BLINK_DURATION = 60; //!< Default value for sMonsterConditionBlinkDuration
    const static int HERO_CONDITION_BLINK_PAUSE = 5;    //!< Default value for sHeroConditionBlinkPause
    const static int MONSTER_CONDITION_BLINK_PAUSE = 5; //!< Default value for sMonsterConditionBlinkPause

    const static int BAR_X_OFFSET = 16;                 //!< X position of a bar element in the display relative to its gauge
    const static int HERO_HEALTH_NUMBER_X_OFFSET = 16;  //!< Default value for sHeroHealthNumberXOffset
    const static int MONSTER_HEALTH_NUMBER_X_OFFSET = 16;   //!< Default value for sMonsterHealthNumberXOffset
    const static int HERO_MANA_NUMBER_X_OFFSET = 16;    //!< Default value for sHeroManaNumberXOffset
    const static int MONSTER_MANA_NUMBER_X_OFFSET = 16; //!< Default value for sMonsterManaNumberXOffset
    const static int HERO_HEALTH_GAUGE_Y_OFFSET = 16;   //!< Default value for sHeroHealthGaugeYOffset
    const static int MONSTER_HEALTH_GAUGE_Y_OFFSET = 16;    //!< Default value for sMonsterHealthGaugeYOffset
    const static int HERO_MANA_GAUGE_Y_OFFSET = 16;     //!< Default value for sHeroManaGaugeYOffset
    const static int MONSTER_MANA_GAUGE_Y_OFFSET = 16;  //!< Default value for sMonsterManaGaugeYOffset
    const static int HERO_ATB_GAUGE_Y_OFFSET = 16;      //!< Default value for sHeroATBGaugeYOffset
    const static int MONSTER_ATB_GAUGE_Y_OFFSET = 16;   //!< Default value for sMonsterATBGaugeYOffset
    const static int CONDITION_Y_OFFSET = 12;           //!< Default value for sConditionYOffset

    const static int DISPLAY_WIDTH = GAUGE_WIDTH + DIGIT_WIDTH;                     //!< Default value of sDisplayWidth (same as the width of a gauge plus one digit for possible overflow of health number)
    const static int DISPLAY_HEIGHT = 3 * GAUGE_HEIGHT + 3 * CONDITION_Y_OFFSET;    //!< Default value of sDisplayHeight (same as the height of the HP, MP, and ATB gauges stacked on top of each other plus three condition icons)

    const static int ATB_MAX = 300000;                  //!< Maximum value for an ATB bar before it's supposed to reset

    //! Horizontal alignment of a battle display
    /*!
        This enumeration is used to distinguish between the different options for horizontally
        aligning the battle display image with the sprite of the Battler it serves.
    */
    enum DisplayHorizontalAlign
    {
        RIGHT_ALIGN_TO_LEFT     = -4,                   //!< Right-aligned against left side of battler sprite
        CENTER_ON_LEFT          = -3,                   //!< Centered on left side of battler sprite
        LEFT_ALIGN_TO_LEFT      = -2,                   //!< Left-aligned against left side of battler sprite
        RIGHT_ALIGN_TO_CENTER   = -1,                   //!< Right-aligned against center of battler sprite
        CENTER_ON_CENTER        =  0,                   //!< Centered on center of battler sprite
        LEFT_ALIGN_TO_CENTER    =  1,                   //!< Left-aligned against center of battler sprite
        RIGHT_ALIGN_TO_RIGHT    =  2,                   //!< Right-aligned against right side of battler sprite
        CENTER_ON_RIGHT         =  3,                   //!< Centered on right side of battler sprite
        LEFT_ALIGN_TO_RIGHT     =  4                    //!< Left-aligned against right side of battler sprite
    };

    //! Vertical alignment of a battle display
    /*!
        This enumeration is used to distinguish between the different options for vertically
        aligning the battle display image with the sprite of the Battler it serves.
    */
    enum DisplayVerticalAlign
    {
        TOP_ALIGN_TO_BOTTOM     = -4,                   //!< Top-aligned against bottom side of battler sprite
        MIDDLE_ON_BOTTOM        = -3,                   //!< Centered on bottom side of battler sprite
        BOTTOM_ALIGN_TO_BOTTOM  = -2,                   //!< Bottom-aligned against bottom side of battler sprite
        TOP_ALIGN_TO_MIDDLE     = -1,                   //!< Top-aligned against middle of battler sprite
        MIDDLE_ON_MIDDLE        =  0,                   //!< Centered on middle of battler sprite
        BOTTOM_ALIGN_TO_MIDDLE  =  1,                   //!< Bottom-aligned against middle of battler sprite
        TOP_ALIGN_TO_TOP        =  2,                   //!< Top-aligned against top side of battler sprite
        MIDDLE_ON_TOP           =  3,                   //!< Centered on top side of battler sprite
        BOTTOM_ALIGN_TO_TOP     =  4                    //!< Bottom-aligned against top side of battler sprite
    };

    //! Bar display mode
    /*!
        This enumeration is used to distinguish between health and mana gauges whose bars fill in
        the typical manner for RPG Maker 2003 (as close as can be managed, anyway) and gauges which
        use an enhanced mode created for DynBattleDisplay.

        Regular: Bar type A for the gauge is used unless it is completely full, in which case bar
        type B is used.
        Enhanced: Bar type B is used to show the current value of the data represented by the gauge,
        while bar type A is used to show the difference between that value and an "anchor" value.
        Whenever there is a change to the current value of the data, the anchor value is set to the
        value the data had until the change. Then the difference portion of the bar is shrunk until
        it disappears. This creates a visual indicator for the player of how much the value has
        changed which can be intuitively compared to the gauge as a whole and the current value.
    */
    enum BarMode
    {
        REGULAR = 0,                                    //!< Bar works as usual for an RM2K3 Gauge display (as close as manageable)
        ENHANCED = 1                                    //!< Bar works in DynBattleDisplay enhanced mode
    };

    static int sDisplayWidth;                           //!< Width of a battle display
    static int sDisplayHeight;                          //!< Height of a battle display
    static DisplayHorizontalAlign sHeroHorizontalDisplayAlign;      //!< Horizontal alignment of the battle display for heroes
    static DisplayHorizontalAlign sMonsterHorizontalDisplayAlign;   //!< Horizontal alignment of the battle display for monsters
    static DisplayVerticalAlign sHeroVerticalDisplayAlign;          //!< Vertical alignment of the battle display for heroes
    static DisplayVerticalAlign sMonsterVerticalDisplayAlign;       //!< Vertical alignment of the battle display for monsters

    static bool sShowHeroATBGauge;                      //!< Whether ATB gauges for heroes should be shown
    static bool sShowMonsterATBGauge;                   //!< Whether ATB gauges for monsters should be shown
    static bool sUseSpecialATB;                         //!< Whether the special ATB bars for "fast" and "slow" should be used
    static bool sHeroShowConditionTurnsRemaining;       //!< Whether the minimum number of turns remaining for a status condition for heroes should be shown
    static bool sMonsterShowConditionTurnsRemaining;    //!< Whether the minimum number of turns remaining for a status condition for monsters should be shown
    static int sHeroHealthGaugeDuration;                //!< Duration (in terms of how many times Update() is called) of a hero health gauge being shown after the value changes (-1 to indicate always showing)
    static int sMonsterHealthGaugeDuration;             //!< Duration (in terms of how many times Update() is called) of a monster health gauge being shown after the value changes (-1 to indicate always showing)
    static int sHeroManaGaugeDuration;                  //!< Duration (in terms of how many times Update() is called) of a hero mana gauge being shown after the value changes (-1 to indicate always showing)
    static int sMonsterManaGaugeDuration;               //!< Duration (in terms of how many times Update() is called) of a monster mana gauge being shown after the value changes (-1 to indicate always showing)
    static bool sHeroShowDisplayWhenSelected;           //!< Whether display elements which are usually shown temporarily should be forced shown when a hero is selected
    static bool sMonsterShowDisplayWhenSelected;        //!< Whether display elements which are usually shown temporarily should be forced shown when a monster is selected
    static BarMode sHeroHealthBarMode;                  //!< Health bar mode for heroes
    static BarMode sMonsterHealthBarMode;               //!< Health bar mode for monsters
    static BarMode sHeroManaBarMode;                    //!< Mana bar mode for heroes
    static BarMode sMonsterManaBarMode;                 //!< Mana bar mode for monsters
    static int sHeroHealthChangePause;                  //!< Pause (in terms of how many times Update() is called) of a hero health bar's difference portion before it starts shrinking
    static int sMonsterHealthChangePause;               //!< Pause (in terms of how many times Update() is called) of a monster health bar's difference portion before it starts shrinking
    static int sHeroManaChangePause;                    //!< Pause (in terms of how many times Update() is called) of a hero mana bar's difference portion before it starts shrinking
    static int sMonsterManaChangePause;                 //!< Pause (in terms of how many times Update() is called) of a monster mana bar's difference portion before it starts shrinking
    static int sHeroHealthChangeDuration;               //!< Duration (in terms of how many times Update() is called) of a hero health bar's difference portion's shrinking animation
    static int sMonsterHealthChangeDuration;            //!< Duration (in terms of how many times Update() is called) of a monster health bar's difference portion's shrinking animation
    static int sHeroManaChangeDuration;                 //!< Duration (in terms of how many times Update() is called) of a hero mana bar's difference portion's shrinking animation
    static int sMonsterManaChangeDuration;              //!< Duration (in terms of how many times Update() is called) of a monster mana bar's difference portion's shrinking animation
    static bool sHeroShowHealthNumber;                  //!< Whether exact number health for heroes should be shown
    static bool sMonsterShowHealthNumber;               //!< Whether exact number health for monsters should be shown
    static bool sHeroShowManaNumber;                    //!< Whether exact number mana for heroes should be shown
    static bool sMonsterShowManaNumber;                 //!< Whether exact number mana for monsters should be shown

    static int sHeroMaxConditionDisplay;                //!< Maximum number of status condition icons to display at a time for a hero
    static int sMonsterMaxConditionDisplay;             //!< Maximum number of status condition icons to display at a time for a monster
    static int sHeroConditionDuration;                  //!< Duration (in terms of how many times Update() is called) of a hero status condition being shown after the value changes (-1 to indicate always showing)
    static int sMonsterConditionDuration;               //!< Duration (in terms of how many times Update() is called) of a monster status condition being shown after the value changes (-1 to indicate always showing)
    static int sHeroConditionBlinkDuration;             //!< Duration (in terms of how many times Update() is called) of a hero status condition blinking after the condition ends
    static int sMonsterConditionBlinkDuration;          //!< Duration (in terms of how many times Update() is called) of a monster status condition blinking after the condition ends
    static int sHeroConditionBlinkPause;                //!< Pause (in terms of how many times Update() is called) of a hero status condition before switching from on to off or vice-versa during blinking animation
    static int sMonsterConditionBlinkPause;             //!< Pause (in terms of how many times Update() is called) of a monster status condition before switching from on to off or vice-versa during blinking animation

    static std::vector<int> sFastConditions;            //!< Vector array of indices of status conditions that are considered "fast"
    static std::vector<int> sSlowConditions;            //!< Vector array of indices of status conditions that are considered "slow"
    static std::vector<int> sIconConditions;            //!< Vector array of indices of status conditions that should be displayed in order of their icon's positions in the icon picture file (also functions as condition priority for displaying)
    static int sNumIconConditions;                      //!< Number of status conditions that have associated icon images (could be less than the size of sIconConditions due to running out of icon images)

    static int sHeroSpriteWidth;                        //!< Width of a hero sprite
    static int sHeroSpriteHeight;                       //!< Height of a hero sprite
    static int sHeroHealthNumberXOffset;                //!< X position of exact number health for a hero
    static int sMonsterHealthNumberXOffset;             //!< X position of exact number health for a monster
    static int sHeroManaNumberXOffset;                  //!< X position of exact number mana for a hero
    static int sMonsterManaNumberXOffset;               //!< X position of exact number mana for a monster
    static int sHeroHealthGaugeYOffset;                 //!< Y position of the next element in the display relative to the health gauge for a hero
    static int sMonsterHealthGaugeYOffset;              //!< Y position of the next element in the display relative to the health gauge for a monster
    static int sHeroManaGaugeYOffset;                   //!< Y position of the next element in the display relative to the mana gauge for a hero
    static int sMonsterManaGaugeYOffset;                //!< Y position of the next element in the display relative to the mana gauge for a monster
    static int sHeroATBGaugeYOffset;                    //!< Y position of the next element in the display relative to the ATB gauge for a hero
    static int sMonsterATBGaugeYOffset;                 //!< Y position of the next element in the display relative to the ATB gauge for a monster
    static int sConditionIconWidth;                     //!< Width of a status condition icon
    static int sConditionIconHeight;                    //!< Height of a status condition icon
    static int sConditionYOffset;                       //!< Y position of the next element in the display relative to a condition element

    static RPG::Image * sHealthGaugePtr;                //!< Pointer to an Image of the health gauge
    static RPG::Image * sManaGaugePtr;                  //!< Pointer to an Image of the mana gauge
    static RPG::Image * sATBGaugePtr;                   //!< Pointer to an Image of the ATB gauge
    static RPG::Image * sHealthBarAPtr;                 //!< Pointer to an Image of health bar A ("non-full")
    static RPG::Image * sHealthBarBPtr;                 //!< Pointer to an Image of health bar B ("full")
    static RPG::Image * sManaBarAPtr;                   //!< Pointer to an Image of mana bar A ("non-full")
    static RPG::Image * sManaBarBPtr;                   //!< Pointer to an Image of mana bar B ("full")
    static RPG::Image * sATBBarAPtr;                    //!< Pointer to an Image of ATB bar A ("non-full")
    static RPG::Image * sATBBarBPtr;                    //!< Pointer to an Image of ATB bar B ("full")
    static RPG::Image * sFastATBBarAPtr;                //!< Pointer to an Image of fast ATB bar A ("non-full")
    static RPG::Image * sFastATBBarBPtr;                //!< Pointer to an Image of fast ATB bar B ("full")
    static RPG::Image * sSlowATBBarAPtr;                //!< Pointer to an Image of slow ATB bar A ("non-full")
    static RPG::Image * sSlowATBBarBPtr;                //!< Pointer to an Image of slow ATB bar B ("full")
    static RPG::Image * sDigitPtr[NUM_DIGITS];          //!< Array of pointers to Image of numerical digits 0-9
    static std::map<int, RPG::Image *> sConditionIconPtr;       //!< Map of pointers to Image of status condition icons
    static RPG::Image * sSystemDisplayPtr;              //!< Pointer to Image used for the parts of the display which come from System2
    static RPG::Image * sIconDisplayPtr;                //!< Pointer to Image used for the parts of the display which come from the icon file (necessary because icons probably use a different palette than System2)

    static int sHeroHorizontalDisplayAlignVar;          //!< Reference of RM2K3 variable for changing sHeroHorizontalDisplayAlign
    static int sMonsterHorizontalDisplayAlignVar;       //!< Reference of RM2K3 variable for changing sMonsterHorizontalDisplayAlign
    static int sHeroVerticalDisplayAlignVar;            //!< Reference of RM2K3 variable for changing sHeroVerticalDisplayAlign
    static int sMonsterVerticalDisplayAlignVar;         //!< Reference of RM2K3 variable for changing sMonsterVerticalDisplayAlign
    static int sShowHeroATBGaugeVar;                    //!< Reference of RM2K3 variable for changing sShowHeroATBGauge
    static int sShowMonsterATBGaugeVar;                 //!< Reference of RM2K3 variable for changing sShowMonsterATBGauge
    static int sUseSpecialATBVar;                       //!< Reference of RM2K3 variable for changing sUseSpecialATB
    static int sHeroShowConditionTurnsRemainingVar;     //!< Reference of RM2K3 variable for changing sHeroShowConditionTurnsRemaining
    static int sMonsterShowConditionTurnsRemainingVar;  //!< Reference of RM2K3 variable for changing sMonsterShowConditionTurnsRemaining
    static int sHeroHealthGaugeDurationVar;             //!< Reference of RM2K3 variable for changing sHeroHealthGaugeDuration
    static int sMonsterHealthGaugeDurationVar;          //!< Reference of RM2K3 variable for changing sMonsterHealthGaugeDuration
    static int sHeroManaGaugeDurationVar;               //!< Reference of RM2K3 variable for changing sHeroManaGaugeDuration
    static int sMonsterManaGaugeDurationVar;            //!< Reference of RM2K3 variable for changing sMonsterManaGaugeDuration
    static int sHeroShowDisplayWhenSelectedVar;         //!< Reference of RM2K3 variable for changing sHeroShowDisplayWhenSelected
    static int sMonsterShowDisplayWhenSelectedVar;      //!< Reference of RM2K3 variable for changing sMonsterShowDisplayWhenSelected
    static int sHeroHealthBarModeVar;                   //!< Reference of RM2K3 variable for changing sHeroHealthBarMode
    static int sMonsterHealthBarModeVar;                //!< Reference of RM2K3 variable for changing sMonsterHealthBarMode
    static int sHeroManaBarModeVar;                     //!< Reference of RM2K3 variable for changing sHeroManaBarMode
    static int sMonsterManaBarModeVar;                  //!< Reference of RM2K3 variable for changing sMonsterManaBarMode
    static int sHeroHealthChangePauseVar;               //!< Reference of RM2K3 variable for changing sHeroHealthChangePause
    static int sMonsterHealthChangePauseVar;            //!< Reference of RM2K3 variable for changing sMonsterHealthChangePause
    static int sHeroManaChangePauseVar;                 //!< Reference of RM2K3 variable for changing sHeroManaChangePause
    static int sMonsterManaChangePauseVar;              //!< Reference of RM2K3 variable for changing sMonsterManaChangePause
    static int sHeroHealthChangeDurationVar;            //!< Reference of RM2K3 variable for changing sHeroHealthChangeDuration
    static int sMonsterHealthChangeDurationVar;         //!< Reference of RM2K3 variable for changing sMonsterHealthChangeDuration
    static int sHeroManaChangeDurationVar;              //!< Reference of RM2K3 variable for changing sHeroManaChangeDuration
    static int sMonsterManaChangeDurationVar;           //!< Reference of RM2K3 variable for changing sMonsterManaChangeDuration
    static int sHeroShowHealthNumberVar;                //!< Reference of RM2K3 variable for changing sHeroShowHealthNumber
    static int sMonsterShowHealthNumberVar;             //!< Reference of RM2K3 variable for changing sMonsterShowHealthNumber
    static int sHeroShowManaNumberVar;                  //!< Reference of RM2K3 variable for changing sHeroShowManaNumber
    static int sMonsterShowManaNumberVar;               //!< Reference of RM2K3 variable for changing sMonsterShowManaNumber
    static int sHeroConditionDurationVar;               //!< Reference of RM2K3 variable for changing sHeroConditionDuration
    static int sMonsterConditionDurationVar;            //!< Reference of RM2K3 variable for changing sMonsterConditionDuration
    static int sHeroConditionBlinkDurationVar;          //!< Reference of RM2K3 variable for changing sHeroConditionBlinkDuration
    static int sMonsterConditionBlinkDurationVar;       //!< Reference of RM2K3 variable for changing sMonsterConditionBlinkDuration
    static int sHeroConditionBlinkPauseVar;             //!< Reference of RM2K3 variable for changing sHeroConditionBlinkPause
    static int sMonsterConditionBlinkPauseVar;          //!< Reference of RM2K3 variable for changing sMonsterConditionBlinkPause

    #ifdef DYNBATTLEDISPLAY_DEBUG
    static std::ofstream * ofDebug;                     //!< Output stream to debug file
    #endif // DYNBATTLEDISPLAY_DEBUG


    //! Default constructor
    /*!
        The default constructor of BattleDisplay provides an empty instance. Make certain this is
        not called until after the RPG class has been initialized (use onInitFinished() for
        earliest).

        \return (BattleDisplay &) Reference to this instance
    */
    BattleDisplay()
    {
        int i;      // Index variable

        // Initialize variables
        mBattlerPtr = NULL;
        mCurHealth = 0;
        mMaxHealth = 0;
        mAnchorHealth = 0;
        mCurMana = 0;
        mMaxMana = 0;
        mAnchorMana = 0;
        mCurATB = 0;
        mHealthGaugeTimer = HERO_HEALTH_GAUGE_DURATION;
        mManaGaugeTimer = HERO_MANA_GAUGE_DURATION;
        mHealthChangeTimer = HERO_HEALTH_CHANGE_DURATION;
        mManaChangeTimer = HERO_MANA_CHANGE_DURATION;
        for( i = 0; i < sNumIconConditions; i++ )
        {
            mCurConditionDuration[sIconConditions[i]] = 0;
            mConditionTimer[sIconConditions[i]] = 0;
        }
    }

    //! Copy constructor
    /*!
        The copy constructor of BattleDisplay copies a given BattleDisplay. Make certain this is
        not called until after the RPG class has been initialized (use onInitFinished() for
        earliest).

        \param copyin : (const BattleDisplay &) Reference to object to be copied.
        \return (BattleDisplay &) Reference to this instance
    */
    BattleDisplay( const BattleDisplay & copyin )
    {
        mBattlerPtr = copyin.mBattlerPtr;
        mCurHealth = copyin.mCurHealth;
        mMaxHealth = copyin.mMaxHealth;
        mAnchorHealth = copyin.mAnchorHealth;
        mCurMana = copyin.mCurMana;
        mMaxMana = copyin.mMaxMana;
        mAnchorMana = copyin.mAnchorMana;
        mCurATB = copyin.mCurATB;
        mHealthGaugeTimer = copyin.mHealthGaugeTimer;
        mManaGaugeTimer = copyin.mManaGaugeTimer;
        mHealthChangeTimer = copyin.mHealthChangeTimer;
        mManaChangeTimer = copyin.mManaChangeTimer;
        mCurConditionDuration = copyin.mCurConditionDuration;
        mConditionTimer = copyin.mConditionTimer;
    }

    //! Constructor
    /*!
        This constructor creates a BattleDisplay for a given Battler. Make certain this is not
        called until after the RPG class has been initialized (use onInitFinished() for earliest).

        \param rBattlerPtr : (RPG::Battler *) Pointer to the Battler which this BattleDisplay will serve
        \return ( BattleDisplay & ) Reference to this instance
    */
    BattleDisplay( RPG::Battler * rBattlerPtr )
    {
        int i;      // Index variable

        // Initialize variables
        mBattlerPtr = rBattlerPtr;
        mCurHealth = mBattlerPtr->hp;
        mMaxHealth = mBattlerPtr->getMaxHp();
        mCurMana = mBattlerPtr->mp;
        mMaxMana = mBattlerPtr->getMaxMp();
        mCurATB = mBattlerPtr->atbValue;
        mHealthGaugeTimer = sHeroHealthGaugeDuration;
        if( mBattlerPtr->isMonster() )
            mHealthGaugeTimer = sMonsterHealthGaugeDuration;
        mManaGaugeTimer = sHeroManaGaugeDuration;
        if( mBattlerPtr->isMonster() )
            mManaGaugeTimer = sMonsterManaGaugeDuration;
        mHealthChangeTimer = sHeroHealthChangeDuration;
        if( mBattlerPtr->isMonster() )
            mHealthChangeTimer = sMonsterHealthChangeDuration;
        mManaChangeTimer = sHeroManaChangeDuration;
        if( mBattlerPtr->isMonster() )
            mManaChangeTimer = sMonsterManaChangeDuration;
        for( i = 0; i < sNumIconConditions; i++ )
        {
            mCurConditionDuration[sIconConditions[i]] = mBattlerPtr->conditions[sIconConditions[i]];
            mConditionTimer[sIconConditions[i]] = 0;
        }
    }

    //! Destructor
    /*!
        The destructor of BattleDisplay deletes all member objects and the BattleDisplay object
        itself.
    */
    ~BattleDisplay()
    {
        // Note that since mBattlerPtr merely points to a Battler object which exists outside of
        // the context of this plugin, it does not have to (nor should it) be deleted/destroyed.
    }

    //! Overloaded = operator
    /*!
        The = operator for this class turns this instance into a deep copy of the right-hand side
        (with the exception that mBattlerPtr, which is only a pointer to an object which exists
        outside the context of this class, is shallow-copied).

        \param rhs : (const BattleDisplay &) Reference to right-hand side
        \return (BattleDisplay &) Reference to this instance
    */
    BattleDisplay& operator=( const BattleDisplay & rhs )
    {
        mBattlerPtr = rhs.mBattlerPtr;
        mCurHealth = rhs.mCurHealth;
        mMaxHealth = rhs.mMaxHealth;
        mAnchorHealth = rhs.mAnchorHealth;
        mCurMana = rhs.mCurMana;
        mMaxMana = rhs.mMaxMana;
        mAnchorMana = rhs.mAnchorMana;
        mCurATB = rhs.mCurATB;
        mHealthGaugeTimer = rhs.mHealthGaugeTimer;
        mManaGaugeTimer = rhs.mManaGaugeTimer;
        mHealthChangeTimer = rhs.mHealthChangeTimer;
        mManaChangeTimer = rhs.mManaChangeTimer;
        mCurConditionDuration = rhs.mCurConditionDuration;
        mConditionTimer = rhs.mConditionTimer;
        return *this;
    }

    //! Overloaded == operator
    /*!
        The == operator for this class returns 1 iff mBattlerPtr for this instance is equal to
        mBattlerPtr for the right-hand side.

        \param rhs : (const BattleDisplay &) Reference to right-hand side
        \return (int) 0 for false, 1 for true
    */
    int operator==( const BattleDisplay &rhs ) const
    {
        if( mBattlerPtr == rhs.mBattlerPtr) return 1;
        return 0;
    }

    //! Overloaded < operator
    /*!
        The < operator for this class returns 1 iff mBattlerPtr for this instance is less than
        mBattlerPtr for the right-hand side.

        \param rhs : (const BattleDisplay &) Reference to right-hand side
        \return (int) 0 for false, 1 for true
    */
    int operator<( const BattleDisplay &rhs ) const
    {
        if( mBattlerPtr < rhs.mBattlerPtr) return 1;
        return 0;
    }

    //! Sets the configuration variables according to DynRPG.ini
    /*!
        This method sets the configuration variables of the class according to the data provided in
        DynRPG.ini. It is called by InitializeStatic() since the configuration variables are part of
        the class's static members. It is also called when the player starts a new game, to ensure
        that any configuration changes that may have occurred while the program is running are reset
        to the initial configuration. It is also called in response to a
        @dynbattledisplay_default_config comment command.

        \param rConfiguration : (std::map<std::string, std::string>) Configuration data from DynRPG.ini
    */
    static void InitializeConfigVariables( std::map<std::string, std::string> rConfiguration )
    {
        int i;                              // Index variable
        std::string field;                  // Storage string for a single field
        std::string delimiter = ",";        // Delimeter separating values in a multi-value field
        size_t pos = 0;                     // Position in the field string

        // Get configuration data
        if( rConfiguration.count( "HeroHorizontalDisplayAlign" ) )
            sHeroHorizontalDisplayAlign = (DisplayHorizontalAlign) atoi( rConfiguration["HeroHorizontalDisplayAlign"].c_str() );
        if( rConfiguration.count( "MonsterHorizontalDisplayAlign" ) )
            sMonsterHorizontalDisplayAlign = (DisplayHorizontalAlign) atoi( rConfiguration["MonsterHorizontalDisplayAlign"].c_str() );
        if( rConfiguration.count( "HeroVerticalDisplayAlign" ) )
            sHeroVerticalDisplayAlign = (DisplayVerticalAlign) atoi( rConfiguration["HeroVerticalDisplayAlign"].c_str() );
        if( rConfiguration.count( "MonsterVerticalDisplayAlign" ) )
            sMonsterVerticalDisplayAlign = (DisplayVerticalAlign) atoi( rConfiguration["MonsterVerticalDisplayAlign"].c_str() );
        if( rConfiguration.count( "HeroSpriteWidth" ) )
            sHeroSpriteWidth = atoi( rConfiguration["HeroSpriteWidth"].c_str() );
        if( rConfiguration.count( "HeroSpriteHeight" ) )
            sHeroSpriteHeight = atoi( rConfiguration["HeroSpriteHeight"].c_str() );
        if( rConfiguration.count( "ShowHeroATBGauge" ) )
            sShowHeroATBGauge = atoi( rConfiguration["ShowHeroATBGauge"].c_str() );
        if( rConfiguration.count( "ShowMonsterATBGauge" ) )
            sShowMonsterATBGauge = atoi( rConfiguration["ShowMonsterATBGauge"].c_str() );
        if( rConfiguration.count( "UseSpecialATB" ) )
            sUseSpecialATB = atoi( rConfiguration["UseSpecialATB"].c_str() );
        if( rConfiguration.count( "ShowHeroConditionTurnsRemaining" ) )
            sHeroShowConditionTurnsRemaining = atoi( rConfiguration["ShowHeroConditionTurnsRemaining"].c_str() );
        if( rConfiguration.count( "ShowMonsterConditionTurnsRemaining" ) )
            sMonsterShowConditionTurnsRemaining = atoi( rConfiguration["ShowMonsterConditionTurnsRemaining"].c_str() );
        if( rConfiguration.count( "HeroHealthGaugeDuration" ) )
            sHeroHealthGaugeDuration = atoi( rConfiguration["HeroHealthGaugeDuration"].c_str() );
        if( rConfiguration.count( "MonsterHealthGaugeDuration" ) )
            sMonsterHealthGaugeDuration = atoi( rConfiguration["MonsterHealthGaugeDuration"].c_str() );
        if( rConfiguration.count( "HeroManaGaugeDuration" ) )
            sHeroManaGaugeDuration = atoi( rConfiguration["HeroManaGaugeDuration"].c_str() );
        if( rConfiguration.count( "MonsterManaGaugeDuration" ) )
            sMonsterManaGaugeDuration = atoi( rConfiguration["MonsterManaGaugeDuration"].c_str() );
        if( rConfiguration.count( "HeroShowDisplayWhenSelected" ) )
            sHeroShowDisplayWhenSelected = atoi( rConfiguration["HeroShowDisplayWhenSelected"].c_str() );
        if( rConfiguration.count( "MonsterShowDisplayWhenSelected" ) )
            sMonsterShowDisplayWhenSelected = atoi( rConfiguration["MonsterShowDisplayWhenSelected"].c_str() );
        if( rConfiguration.count( "HeroHealthBarMode" ) )
            sHeroHealthBarMode = (BarMode) atoi( rConfiguration["HeroHealthBarMode"].c_str() );
        if( rConfiguration.count( "MonsterHealthBarMode" ) )
            sMonsterHealthBarMode = (BarMode) atoi( rConfiguration["MonsterHealthBarMode"].c_str() );
        if( rConfiguration.count( "HeroManaBarMode" ) )
            sHeroManaBarMode = (BarMode) atoi( rConfiguration["HeroManaBarMode"].c_str() );
        if( rConfiguration.count( "MonsterManaBarMode" ) )
            sMonsterManaBarMode = (BarMode) atoi( rConfiguration["MonsterManaBarMode"].c_str() );
        if( rConfiguration.count( "HeroHealthChangePause" ) )
            sHeroHealthChangePause = atoi( rConfiguration["HeroHealthChangePause"].c_str() );
        if( rConfiguration.count( "MonsterHealthChangePause" ) )
            sMonsterHealthChangePause = atoi( rConfiguration["MonsterHealthChangePause"].c_str() );
        if( rConfiguration.count( "HeroManaChangePause" ) )
            sHeroManaChangePause = atoi( rConfiguration["HeroManaChangePause"].c_str() );
        if( rConfiguration.count( "MonsterManaChangePause" ) )
            sMonsterManaChangePause = atoi( rConfiguration["MonsterManaChangePause"].c_str() );
        if( rConfiguration.count( "HeroHealthChangeDuration" ) )
            sHeroHealthChangeDuration = atoi( rConfiguration["HeroHealthChangeDuration"].c_str() );
        if( rConfiguration.count( "MonsterHealthChangeDuration" ) )
            sMonsterHealthChangeDuration = atoi( rConfiguration["MonsterHealthChangeDuration"].c_str() );
        if( rConfiguration.count( "HeroManaChangeDuration") )
            sHeroManaChangeDuration = atoi( rConfiguration["HeroManaChangeDuration"].c_str() );
        if( rConfiguration.count( "MonsterManaChangeDuration" ) )
            sMonsterManaChangeDuration = atoi( rConfiguration["MonsterManaChangeDuration"].c_str() );
        if( rConfiguration.count( "HeroShowHealthNumber" ) )
            sHeroShowHealthNumber = atoi( rConfiguration["HeroShowHealthNumber"].c_str() );
        if( rConfiguration.count( "MonsterShowHealthNumber" ) )
            sMonsterShowHealthNumber = atoi( rConfiguration["MonsterShowHealthNumber"].c_str() );
        if( rConfiguration.count( "HeroShowManaNumber" ) )
            sHeroShowManaNumber = atoi( rConfiguration["HeroShowManaNumber"].c_str() );
        if( rConfiguration.count( "MonsterShowManaNumber" ) )
            sMonsterShowManaNumber = atoi( rConfiguration["MonsterShowManaNumber"].c_str() );
        if( rConfiguration.count( "HeroHealthNumberXOffset" ) )
            sHeroHealthNumberXOffset = atoi( rConfiguration["HeroHealthNumberXOffset"].c_str() );
        if( rConfiguration.count( "MonsterHealthNumberXOffset" ) )
            sMonsterHealthNumberXOffset = atoi( rConfiguration["MonsterHealthNumberXOffset"].c_str() );
        if( rConfiguration.count( "HeroManaNumberXOffset" ) )
            sHeroManaNumberXOffset = atoi( rConfiguration["HeroManaNumberXOffset"].c_str() );
        if( rConfiguration.count( "MonsterManaNumberXOffset" ) )
            sMonsterManaNumberXOffset = atoi( rConfiguration["MonsterManaNumberXOffset"].c_str() );
        if( rConfiguration.count( "HeroHealthGaugeYOffset" ) )
            sHeroHealthGaugeYOffset = atoi( rConfiguration["HeroHealthGaugeYOffset"].c_str() );
        if( rConfiguration.count( "MonsterHealthGaugeYOffset" ) )
            sMonsterHealthGaugeYOffset = atoi( rConfiguration["MonsterHealthGaugeYOffset"].c_str() );
        if( rConfiguration.count( "HeroManaGaugeYOffset" ) )
            sHeroManaGaugeYOffset = atoi( rConfiguration["HeroManaGaugeYOffset"].c_str() );
        if( rConfiguration.count( "MonsterManaGaugeYOffset" ) )
            sMonsterManaGaugeYOffset = atoi( rConfiguration["MonsterManaGaugeYOffset"].c_str() );
        if( rConfiguration.count( "HeroATBGaugeYOffset" ) )
            sHeroATBGaugeYOffset = atoi( rConfiguration["HeroATBGaugeYOffset"].c_str() );
        if( rConfiguration.count( "MonsterATBGaugeYOffset" ) )
            sMonsterATBGaugeYOffset = atoi( rConfiguration["MonsterATBGaugeYOffset"].c_str() );

        // Only do configuration for status condition icons if the initialization file provides a picture file path for icons
        if( rConfiguration.count( "ConditionIconFile" ) )
        {   // Picture file provided
            if( rConfiguration.count( "ConditionIconWidth" ) )
                sConditionIconWidth = atoi( rConfiguration["ConditionIconWidth"].c_str() );
            if( rConfiguration.count( "ConditionIconHeight" ) )
                sConditionIconHeight = atoi( rConfiguration["ConditionIconHeight"].c_str() );
            if( rConfiguration.count( "ConditionYOffset" ) )
                sConditionYOffset = atoi( rConfiguration["ConditionYOffset"].c_str() );
            sIconConditions.clear();
            if( rConfiguration.count( "IconConditions" ) )
            {   // IconConditions field exists in initialization file
                // Initialize sIconConditions according to provided data
                field = rConfiguration["IconConditions"];
                while( ( pos = field.find( delimiter ) ) != std::string::npos )
                {
                    i = atoi( field.substr( 0, pos ).c_str() );
                    sIconConditions.push_back( i );
                    field.erase( 0, pos + delimiter.length() );
                }
                i = atoi( field.c_str() );
                sIconConditions.push_back( i );
            }
            if( 1 == sIconConditions.size() && sIconConditions[0] == 0 )
            {
                // Assume all status conditions should be represented by icons in the order they exist in the database
                sIconConditions.pop_back();     // Get rid of the 0 currently in sIconConditions
                for( i = 1; i < RPG::conditions.count() + 1; i++ )
                {
                    sIconConditions.push_back( i );
                }
            }
            if( rConfiguration.count( "HeroMaxConditionDisplay" ) )
                sHeroMaxConditionDisplay = atoi( rConfiguration["HeroMaxConditionDisplay"].c_str() );
            if( rConfiguration.count( "MonsterMaxConditionDisplay" ) )
                sMonsterMaxConditionDisplay = atoi( rConfiguration["MonsterMaxConditionDisplay"].c_str() );
            if( rConfiguration.count( "HeroConditionDuration" ) )
                sHeroConditionDuration = atoi( rConfiguration["HeroConditionDuration"].c_str() );
            if( rConfiguration.count( "MonsterConditionDuration" ) )
                sMonsterConditionDuration = atoi( rConfiguration["MonsterConditionDuration"].c_str() );
            if( rConfiguration.count( "HeroConditionBlinkDuration" ) )
                sHeroConditionBlinkDuration = atoi( rConfiguration["HeroConditionBlinkDuration"].c_str() );
            if( rConfiguration.count( "MonsterConditionBlinkDuration" ) )
                sMonsterConditionBlinkDuration = atoi( rConfiguration["MonsterConditionBlinkDuration"].c_str() );
            if( rConfiguration.count( "HeroConditionBlinkPause" ) )
                sHeroConditionBlinkPause = atoi( rConfiguration["HeroConditionBlinkPause"].c_str() );
            if( rConfiguration.count( "MonsterConditionBlinkPause" ) )
                sMonsterConditionBlinkPause = atoi( rConfiguration["MonsterConditionBlinkPause"].c_str() );
        }
        else
        {   // Picture file not provided
            // Ensure that status condition icons are never shown
            sIconConditions.clear();
            sConditionIconWidth = 0;
            sConditionIconHeight = 0;
            sConditionYOffset = 0;
            sHeroMaxConditionDisplay = 0;
            sMonsterMaxConditionDisplay = 0;
            sHeroConditionDuration = 0;
            sMonsterConditionDuration = 0;
            sHeroConditionBlinkDuration = 0;
            sMonsterConditionBlinkDuration = 0;
        }

        // Get indices of RM2K3 variables for changing configuration values at run-time and set the
        // RM2K3 variables according to current configuration values
        if( rConfiguration.count( "HeroHorizontalDisplayAlignVar" ) )
            sHeroHorizontalDisplayAlignVar = atoi( rConfiguration["HeroHorizontalDisplayAlignVar"].c_str() );
        if( rConfiguration.count( "MonsterHorizontalDisplayAlignVar" ) )
            sMonsterHorizontalDisplayAlignVar = atoi( rConfiguration["MonsterHorizontalDisplayAlignVar"].c_str() );
        if( rConfiguration.count( "HeroVerticalDisplayAlignVar" ) )
            sHeroVerticalDisplayAlignVar = atoi( rConfiguration["HeroVerticalDisplayAlignVar"].c_str() );
        if( rConfiguration.count( "MonsterVerticalDisplayAlignVar" ) )
            sMonsterVerticalDisplayAlignVar = atoi( rConfiguration["MonsterVerticalDisplayAlignVar"].c_str() );
        if( rConfiguration.count( "ShowHeroATBGaugeVar" ) )
            sShowHeroATBGaugeVar = atoi( rConfiguration["ShowHeroATBGaugeVar"].c_str() );
        if( rConfiguration.count( "ShowMonsterATBGaugeVar" ) )
            sShowMonsterATBGaugeVar = atoi( rConfiguration["ShowMonsterATBGaugeVar"].c_str() );
        if( rConfiguration.count( "UseSpecialATBVar" ) )
            sUseSpecialATBVar = atoi( rConfiguration["UseSpecialATBVar"].c_str() );
        if( rConfiguration.count( "HeroShowConditionTurnsRemainingVar" ) )
            sHeroShowConditionTurnsRemainingVar = atoi( rConfiguration["HeroShowConditionTurnsRemainingVar"].c_str() );
        if( rConfiguration.count( "MonsterShowConditionTurnsRemainingVar" ) )
            sMonsterShowConditionTurnsRemainingVar = atoi( rConfiguration["MonsterShowConditionTurnsRemainingVar"].c_str() );
        if( rConfiguration.count( "HeroHealthGaugeDurationVar" ) )
            sHeroHealthGaugeDurationVar = atoi( rConfiguration["HeroHealthGaugeDurationVar"].c_str() );
        if( rConfiguration.count( "MonsterHealthGaugeDurationVar" ) )
            sMonsterHealthGaugeDurationVar = atoi( rConfiguration["MonsterHealthGaugeDurationVar"].c_str() );
        if( rConfiguration.count( "HeroManaGaugeDurationVar" ) )
            sHeroManaGaugeDurationVar = atoi( rConfiguration["HeroManaGaugeDurationVar"].c_str() );
        if( rConfiguration.count( "MonsterManaGaugeDurationVar" ) )
            sMonsterManaGaugeDurationVar = atoi( rConfiguration["MonsterManaGaugeDurationVar"].c_str() );
        if( rConfiguration.count( "HeroShowDisplayWhenSelectedVar" ) )
            sHeroShowDisplayWhenSelectedVar = atoi( rConfiguration["HeroShowDisplayWhenSelectedVar"].c_str() );
        if( rConfiguration.count( "MonsterShowDisplayWhenSelectedVar" ) )
            sMonsterShowDisplayWhenSelectedVar = atoi( rConfiguration["MonsterShowDisplayWhenSelectedVar"].c_str() );
        if( rConfiguration.count( "HeroHealthBarModeVar" ) )
            sHeroHealthBarModeVar = atoi( rConfiguration["HeroHealthBarModeVar"].c_str() );
        if( rConfiguration.count( "MonsterHealthBarModeVar" ) )
            sMonsterHealthBarModeVar = atoi( rConfiguration["MonsterHealthBarModeVar"].c_str() );
        if( rConfiguration.count( "HeroManaBarModeVar" ) )
            sHeroManaBarModeVar = atoi( rConfiguration["HeroManaBarModeVar"].c_str() );
        if( rConfiguration.count( "MonsterManaBarModeVar" ) )
            sMonsterManaBarModeVar = atoi( rConfiguration["MonsterManaBarModeVar"].c_str() );
        if( rConfiguration.count( "HeroHealthChangePauseVar" ) )
            sHeroHealthChangePauseVar = atoi( rConfiguration["HeroHealthChangePauseVar"].c_str() );
        if( rConfiguration.count( "MonsterHealthChangePauseVar" ) )
            sMonsterHealthChangePauseVar = atoi( rConfiguration["MonsterHealthChangePauseVar"].c_str() );
        if( rConfiguration.count( "HeroManaChangePauseVar" ) )
            sHeroManaChangePauseVar = atoi( rConfiguration["HeroManaChangePauseVar"].c_str() );
        if( rConfiguration.count( "MonsterManaChangePauseVar" ) )
            sMonsterManaChangePauseVar = atoi( rConfiguration["MonsterManaChangePauseVar"].c_str() );
        if( rConfiguration.count( "HeroHealthChangeDurationVar" ) )
            sHeroHealthChangeDurationVar = atoi( rConfiguration["HeroHealthChangeDurationVar"].c_str() );
        if( rConfiguration.count( "MonsterHealthChangeDurationVar" ) )
            sMonsterHealthChangeDurationVar = atoi( rConfiguration["MonsterHealthChangeDurationVar"].c_str() );
        if( rConfiguration.count( "HeroManaChangeDurationVar" ) )
            sHeroManaChangeDurationVar = atoi( rConfiguration["HeroManaChangeDurationVar"].c_str() );
        if( rConfiguration.count( "MonsterManaChangeDurationVar" ) )
            sMonsterManaChangeDurationVar = atoi( rConfiguration["MonsterManaChangeDurationVar"].c_str() );
        if( rConfiguration.count( "HeroShowHealthNumberVar" ) )
            sHeroShowHealthNumberVar = atoi( rConfiguration["HeroShowHealthNumberVar"].c_str() );
        if( rConfiguration.count( "MonsterShowHealthNumberVar" ) )
            sMonsterShowHealthNumberVar = atoi( rConfiguration["MonsterShowHealthNumberVar"].c_str() );
        if( rConfiguration.count( "HeroShowManaNumberVar" ) )
            sHeroShowManaNumberVar = atoi( rConfiguration["HeroShowManaNumberVar"].c_str() );
        if( rConfiguration.count( "MonsterShowManaNumberVar" ) )
            sMonsterShowManaNumberVar = atoi( rConfiguration["MonsterShowManaNumberVar"].c_str() );
        if( rConfiguration.count( "HeroConditionDurationVar" ) )
            sHeroConditionDurationVar = atoi( rConfiguration["HeroConditionDurationVar"].c_str() );
        if( rConfiguration.count( "MonsterConditionDurationVar" ) )
            sMonsterConditionDurationVar = atoi( rConfiguration["MonsterConditionDurationVar"].c_str() );
        if( rConfiguration.count( "HeroConditionBlinkDurationVar" ) )
            sHeroConditionBlinkDurationVar = atoi( rConfiguration["HeroConditionBlinkDurationVar"].c_str() );
        if( rConfiguration.count( "MonsterConditionBlinkDurationVar" ) )
            sMonsterConditionBlinkDurationVar = atoi( rConfiguration["MonsterConditionBlinkDurationVar"].c_str() );
        if( rConfiguration.count( "HeroConditionBlinkPauseVar" ) )
            sHeroConditionBlinkPauseVar = atoi( rConfiguration["HeroConditionBlinkPauseVar"].c_str() );
        if( rConfiguration.count( "MonsterConditionBlinkPauseVar" ) )
            sMonsterConditionBlinkPauseVar = atoi( rConfiguration["MonsterConditionBlinkPauseVar"].c_str() );
    }

    //! Initializes static member variables
    /*!
        This method initializes the static member variables of the class. It should be called
        before any instances of the class are created. Make certain this is not called until after
        the RPG class has been initialized (use onInitFinished() for earliest).

        \param rConfiguration : (std::map<std::string, std::string>) Configuration data from DynRPG.ini
    */
    static void InitializeStatic( std::map<std::string, std::string> rConfiguration )
    {
        int i;                              // Index variable
        int curX, curY;                     // Coordinates, used to pull individual icons from a provided image
        std::string field;                  // Storage string for a single field
        std::string delimiter = ",";        // Delimeter separating values in a multi-value field
        RPG::Image * system2ImagePtr;        // Pointer to Image used as source of display elements; could come from user-specified picture file or default System2 graphics
        RPG::Image * allIconsImagePtr = RPG::Image::create();       // Pointer to temporary Image, used for holding the contents of the icons picture file
        RPG::Image * newImagePtr;           // Pointer to a new Image, used for generating icon Images before storing in sConditionIconPtr

        // Initialize configuration variables
        InitializeConfigVariables( rConfiguration );

        // Determine the needed size for battle display images
        sDisplayWidth = DISPLAY_WIDTH;
        if( sDisplayWidth < sHeroHealthNumberXOffset + 5 * DIGIT_WIDTH )
        {   // The exact number health display for heroes takes more than the current display width
            sDisplayWidth = sHeroHealthNumberXOffset + 5 * DIGIT_WIDTH;
        }
        if( sDisplayWidth < sMonsterHealthNumberXOffset + 5 * DIGIT_WIDTH )
        {   // The exact number health display for monsters takes more than the current display width
            sDisplayWidth = sMonsterHealthNumberXOffset + 5 * DIGIT_WIDTH;
        }
        if( sDisplayWidth < sHeroManaNumberXOffset + 3 * DIGIT_WIDTH )
        {   // The exact number mana display for heroes takes more than the current display width
            sDisplayWidth = sHeroManaNumberXOffset + 3 * DIGIT_WIDTH;
        }
        if( sDisplayWidth < sMonsterManaNumberXOffset + 3 * DIGIT_WIDTH )
        {   // The exact number mana display for monsters takes more than the current display width
            sDisplayWidth = sMonsterManaNumberXOffset + 3 * DIGIT_WIDTH;
        }
        if( sDisplayWidth < sConditionIconWidth + 4 * DIGIT_WIDTH )
        {   // The condition icons plus possible digits take more than the current display width
            sDisplayWidth = sConditionIconWidth + 4 * DIGIT_WIDTH;
        }
        sDisplayHeight = sHeroHealthGaugeYOffset + sHeroManaGaugeYOffset + sHeroATBGaugeYOffset     // Assume all three gauges may be shown at one time
            + sHeroMaxConditionDisplay * sConditionIconHeight;
        if( sDisplayHeight < sMonsterHealthGaugeYOffset + sMonsterManaGaugeYOffset + sMonsterATBGaugeYOffset
            + sMonsterMaxConditionDisplay * sConditionIconHeight )
        {   // Monsters' possible display elements take more than the current display height
            sDisplayHeight = sMonsterHealthGaugeYOffset + sMonsterManaGaugeYOffset + sMonsterATBGaugeYOffset        // Assume all three gauges may be shown at one time
                + sMonsterMaxConditionDisplay * sConditionIconHeight;
        }

        // Make arrays of status conditions which double or halve agility
        for( i = 1; i < RPG::conditions.count() + 1; i++ )      // The RPG::conditions container begins at index 1
        {
            if( RPG::conditions[i]->alterAgi )
            {   // Condition i alters agility
                if( RPG::BSA_DOUBLE == RPG::conditions[i]->statAlteration )
                {   // Condition i doubles agility
                    sFastConditions.push_back( i );
                }
                if( RPG::BSA_HALF == RPG::conditions[i]->statAlteration )
                {   // Condition i halves agility
                    sSlowConditions.push_back( i );
                }
            }
        }

        // Initialize images
        if( rConfiguration.count( "System2File" ) )
        {   // System2 file provided
            system2ImagePtr = RPG::Image::create();
            system2ImagePtr->loadFromFile( rConfiguration["System2File"].c_str() );
        }
        else
        {   // System2 file not provided, use default
            system2ImagePtr = RPG::system->systemGraphic->system2Image;
        }
        sHealthGaugePtr->useMaskColor = true;
        sHealthGaugePtr->setPalette( system2ImagePtr->palette );
        sHealthGaugePtr->draw( 0, 0,                                            // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               HEALTH_GAUGE_SRC_X, HEALTH_GAUGE_SRC_Y,          // Coordinates in source Image
                               GAUGE_WIDTH, GAUGE_HEIGHT,                       // Dimensions in source Image
                               0 );                                             // Transparency color
        sManaGaugePtr->useMaskColor = true;
        sManaGaugePtr->setPalette( system2ImagePtr->palette );
        sManaGaugePtr->draw( 0, 0,                                              // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               MANA_GAUGE_SRC_X, MANA_GAUGE_SRC_Y,              // Coordinates in source Image
                               GAUGE_WIDTH, GAUGE_HEIGHT,                       // Dimensions in source Image
                               0 );                                             // Transparency color
        sATBGaugePtr->useMaskColor = true;
        sATBGaugePtr->setPalette( system2ImagePtr->palette );
        sATBGaugePtr->draw( 0, 0,                                               // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               ATB_GAUGE_SRC_X, ATB_GAUGE_SRC_Y,                // Coordinates in source Image
                               GAUGE_WIDTH, GAUGE_HEIGHT,                       // Dimensions in source Image
                               0 );                                             // Transparency color
        sHealthBarAPtr->useMaskColor = true;
        sHealthBarAPtr->setPalette( system2ImagePtr->palette );
        sHealthBarAPtr->draw( 0, 0,                                             // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               HEALTH_BAR_A_SRC_X, HEALTH_BAR_A_SRC_Y,          // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        sManaBarAPtr->useMaskColor = true;
        sManaBarAPtr->setPalette( system2ImagePtr->palette );
        sManaBarAPtr->draw( 0, 0,                                               // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               MANA_BAR_A_SRC_X, MANA_BAR_A_SRC_Y,              // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        sATBBarAPtr->useMaskColor = true;
        sATBBarAPtr->setPalette( system2ImagePtr->palette );
        sATBBarAPtr->draw( 0, 0,                                                // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               ATB_BAR_A_SRC_X, ATB_BAR_A_SRC_Y,                // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        sHealthBarBPtr->useMaskColor = true;
        sHealthBarBPtr->setPalette( system2ImagePtr->palette );
        sHealthBarBPtr->draw( 0, 0,                                             // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               HEALTH_BAR_B_SRC_X, HEALTH_BAR_B_SRC_Y,          // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        sManaBarBPtr->useMaskColor = true;
        sManaBarBPtr->setPalette( system2ImagePtr->palette );
        sManaBarBPtr->draw( 0, 0,                                               // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               MANA_BAR_B_SRC_X, MANA_BAR_B_SRC_Y,              // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        sATBBarBPtr->useMaskColor = true;
        sATBBarBPtr->setPalette( system2ImagePtr->palette );
        sATBBarBPtr->draw( 0, 0,                                                // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               ATB_BAR_B_SRC_X, ATB_BAR_B_SRC_Y,                // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        sFastATBBarAPtr->useMaskColor = true;
        sFastATBBarAPtr->setPalette( system2ImagePtr->palette );
        sFastATBBarAPtr->draw( 0, 0,                                            // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               FAST_ATB_BAR_A_SRC_X, FAST_ATB_BAR_A_SRC_Y,      // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        sFastATBBarBPtr->useMaskColor = true;
        sFastATBBarBPtr->setPalette( system2ImagePtr->palette );
        sFastATBBarBPtr->draw( 0, 0,                                            // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               FAST_ATB_BAR_B_SRC_X, FAST_ATB_BAR_B_SRC_Y,      // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        sSlowATBBarAPtr->useMaskColor = true;
        sSlowATBBarAPtr->setPalette( system2ImagePtr->palette );
        sSlowATBBarAPtr->draw( 0, 0,                                            // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               SLOW_ATB_BAR_A_SRC_X, SLOW_ATB_BAR_A_SRC_Y,      // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        sSlowATBBarBPtr->useMaskColor = true;
        sSlowATBBarBPtr->setPalette( system2ImagePtr->palette );
        sSlowATBBarBPtr->draw( 0, 0,                                            // Coordinates in destination Image
                               system2ImagePtr,                                 // Source Image pointer
                               SLOW_ATB_BAR_B_SRC_X, SLOW_ATB_BAR_B_SRC_Y,      // Coordinates in source Image
                               BAR_WIDTH, BAR_HEIGHT,                           // Dimensions in source Image
                               0 );                                             // Transparency color
        for( i = 0; i < NUM_DIGITS; i++ )
        {
            sDigitPtr[i]->useMaskColor = true;
            sDigitPtr[i]->setPalette( system2ImagePtr->palette );
            sDigitPtr[i]->draw( 0, 0,                                           // Coordinates in destination Image
                                system2ImagePtr,                                // Source Image pointer
                                DIGIT_SRC_X + DIGIT_WIDTH * i, DIGIT_SRC_Y,     // Coordinates in source Image
                                DIGIT_WIDTH, DIGIT_HEIGHT,                      // Dimensions in source Image
                                0 );                                            // Transparency color
        }
        // Initialize the Image that will be used for drawing individual Battlers' gauges
        sSystemDisplayPtr = RPG::Image::create( sDisplayWidth, sDisplayHeight );
        sSystemDisplayPtr->useMaskColor = true;
        sSystemDisplayPtr->setPalette( system2ImagePtr->palette );
        if( rConfiguration.count( "System2File" ) )
        {   // System2 file was provided and loaded; destroy the Image once it has served its purpose
            RPG::Image::destroy( system2ImagePtr );
        }
        // Only do images for status condition icons if the initialization file provides a picture file path for icons
        if( rConfiguration.count( "ConditionIconFile" ) )
        {   // Picture file provided
            allIconsImagePtr->loadFromFile( rConfiguration["ConditionIconFile"].c_str() );
            curX = 0;
            curY = 0;
            i = 0;
            // Stop adding icon Images after there is one for each showable condition or the end of the source Image is reached, whichever comes first
            while( i < (signed int) sIconConditions.size() && curY + sConditionIconHeight <= allIconsImagePtr->height )
            {
                newImagePtr = RPG::Image::create( sConditionIconWidth, sConditionIconHeight );
                newImagePtr->useMaskColor = true;
                newImagePtr->setPalette( allIconsImagePtr->palette );
                newImagePtr->draw( 0, 0,                                            // Coordinates in destination Image
                                   allIconsImagePtr,                                // Source Image pointer
                                   curX, curY,                                      // Coordinates in source Image
                                   sConditionIconWidth, sConditionIconHeight,       // Dimensions in source Image
                                   0 );                                             // Transparency color
                sConditionIconPtr[sIconConditions[i]] = newImagePtr;
                curX += sConditionIconWidth;
                if( curX + sConditionIconWidth > allIconsImagePtr->width )
                {   // Reached the end of a row of icons
                    curX = 0;
                    curY += sConditionIconHeight;
                }
                i++;
            }
            // Record the number of icons available for conditions (needed because it could be less
            // than the size of sIconConditions
            sNumIconConditions = i;
            // Initialize the Image that will be used for drawing individual Battlers' condition icons
            sIconDisplayPtr = RPG::Image::create( sDisplayWidth, sDisplayHeight );
            sIconDisplayPtr->useMaskColor = true;
            sIconDisplayPtr->setPalette( allIconsImagePtr->palette );
            // Destroy the Image holding the contents of the icons picture file once it has served its purpose
            RPG::Image::destroy( allIconsImagePtr );
        }
    }

    //! Update configuration variables according to the values of reserved RM2K3 variables
    /*!
        This method updates the values of static configuration variables to match the RM2K3
        variables which the user specified for controlling configuration at run-time. It is called
        in response to a @dynbattledisplay_update comment command or to the player loading a
        savestate. Configuration variables without a valid RM2K3 variable index associated with them
        are left alone.
    */
    static void ConfigUpdate()
    {
        if( 0 < sHeroHorizontalDisplayAlignVar )
            sHeroHorizontalDisplayAlign = (DisplayHorizontalAlign) RPG::variables[sHeroHorizontalDisplayAlignVar];
        if( 0 < sMonsterHorizontalDisplayAlignVar )
            sMonsterHorizontalDisplayAlign = (DisplayHorizontalAlign) RPG::variables[sMonsterHorizontalDisplayAlignVar];
        if( 0 < sHeroVerticalDisplayAlignVar )
            sHeroVerticalDisplayAlign = (DisplayVerticalAlign) RPG::variables[sHeroVerticalDisplayAlignVar];
        if( 0 < sMonsterVerticalDisplayAlignVar )
            sMonsterVerticalDisplayAlign = (DisplayVerticalAlign) RPG::variables[sMonsterVerticalDisplayAlignVar];
        if( 0 < sShowHeroATBGaugeVar )
            sShowHeroATBGauge = RPG::variables[sShowHeroATBGaugeVar];
        if( 0 < sShowMonsterATBGaugeVar )
            sShowMonsterATBGauge = RPG::variables[sShowMonsterATBGaugeVar];
        if( 0 < sUseSpecialATBVar )
            sUseSpecialATB = RPG::variables[sUseSpecialATBVar];
        if( 0 < sHeroShowConditionTurnsRemainingVar )
            sHeroShowConditionTurnsRemaining = RPG::variables[sHeroShowConditionTurnsRemainingVar];
        if( 0 < sMonsterShowConditionTurnsRemainingVar )
            sMonsterShowConditionTurnsRemaining = RPG::variables[sMonsterShowConditionTurnsRemainingVar];
        if( 0 < sHeroHealthGaugeDurationVar )
            sHeroHealthGaugeDuration = RPG::variables[sHeroHealthGaugeDurationVar];
        if( 0 < sMonsterHealthGaugeDurationVar )
            sMonsterHealthGaugeDuration = RPG::variables[sMonsterHealthGaugeDurationVar];
        if( 0 < sHeroManaGaugeDurationVar )
            sHeroManaGaugeDuration = RPG::variables[sHeroManaGaugeDurationVar];
        if( 0 < sMonsterManaGaugeDurationVar )
            sMonsterManaGaugeDuration = RPG::variables[sMonsterManaGaugeDurationVar];
        if( 0 < sHeroShowDisplayWhenSelectedVar )
            sHeroShowDisplayWhenSelected = RPG::variables[sHeroShowDisplayWhenSelectedVar];
        if( 0 < sMonsterShowDisplayWhenSelectedVar )
            sMonsterShowDisplayWhenSelected = RPG::variables[sMonsterShowDisplayWhenSelectedVar];
        if( 0 < sHeroHealthBarModeVar )
            sHeroHealthBarMode = (BarMode) RPG::variables[sHeroHealthBarModeVar];
        if( 0 < sMonsterHealthBarModeVar )
            sMonsterHealthBarMode = (BarMode) RPG::variables[sMonsterHealthBarModeVar];
        if( 0 < sHeroManaBarModeVar )
            sHeroManaBarMode = (BarMode) RPG::variables[sHeroManaBarModeVar];
        if( 0 < sMonsterManaBarModeVar )
            sMonsterManaBarMode = (BarMode) RPG::variables[sMonsterManaBarModeVar];
        if( 0 < sHeroHealthChangePauseVar )
            sHeroHealthChangePause = RPG::variables[sHeroHealthChangePauseVar];
        if( 0 < sMonsterHealthChangePauseVar )
            sMonsterHealthChangePause = RPG::variables[sMonsterHealthChangePauseVar];
        if( 0 < sHeroManaChangePauseVar )
            sHeroManaChangePause = RPG::variables[sHeroManaChangePauseVar];
        if( 0 < sMonsterManaChangePauseVar )
            sMonsterManaChangePause = RPG::variables[sMonsterManaChangePauseVar];
        if( 0 < sHeroHealthChangeDurationVar )
            sHeroHealthChangeDuration = RPG::variables[sHeroHealthChangeDurationVar];
        if( 0 < sMonsterHealthChangeDurationVar )
            sMonsterHealthChangeDuration = RPG::variables[sMonsterHealthChangeDurationVar];
        if( 0 < sHeroManaChangeDurationVar )
            sHeroManaChangeDuration = RPG::variables[sHeroManaChangeDurationVar];
        if( 0 < sMonsterManaChangeDurationVar )
            sMonsterManaChangeDuration = RPG::variables[sMonsterManaChangeDurationVar];
        if( 0 < sHeroShowHealthNumberVar )
            sHeroShowHealthNumber = RPG::variables[sHeroShowHealthNumberVar];
        if( 0 < sMonsterShowHealthNumberVar )
            sMonsterShowHealthNumber = RPG::variables[sMonsterShowHealthNumberVar];
        if( 0 < sHeroShowManaNumberVar )
            sHeroShowManaNumber = RPG::variables[sHeroShowManaNumberVar];
        if( 0 < sMonsterShowManaNumberVar )
            sMonsterShowManaNumber = RPG::variables[sMonsterShowManaNumberVar];
        if( 0 < sHeroConditionDurationVar )
            sHeroConditionDuration = RPG::variables[sHeroConditionDurationVar];
        if( 0 < sMonsterConditionDurationVar )
            sMonsterConditionDuration = RPG::variables[sMonsterConditionDurationVar];
        if( 0 < sHeroConditionBlinkDurationVar )
            sHeroConditionBlinkDuration = RPG::variables[sHeroConditionBlinkDurationVar];
        if( 0 < sMonsterConditionBlinkDurationVar )
            sMonsterConditionBlinkDuration = RPG::variables[sMonsterConditionBlinkDurationVar];
        if( 0 < sHeroConditionBlinkPauseVar )
            sHeroConditionBlinkPause = RPG::variables[sHeroConditionBlinkPauseVar];
        if( 0 < sMonsterConditionBlinkPauseVar )
            sMonsterConditionBlinkPause = RPG::variables[sMonsterConditionBlinkPauseVar];
    }

    //! Set the values of reserved RM2K3 variables to configuration data
    /*!
        This method updates the values of the RM2K3 variables which the user specified for
        controlling configuration at run-time to match the static configuration variables. It is
        called when the player starts a new game, to initialize the RM2K3 controller variables
        according to the initial configuration as specified in DynRPG.ini. It is also called in
        response to a @dynbattledisplay_default_config comment command.
    */
    static void SetControllerVariables()
    {
        if( 0 < sHeroHorizontalDisplayAlignVar )
            RPG::variables[sHeroHorizontalDisplayAlignVar] = (int) sHeroHorizontalDisplayAlign;
        if( 0 < sMonsterHorizontalDisplayAlignVar )
            RPG::variables[sMonsterHorizontalDisplayAlignVar] = (int) sMonsterHorizontalDisplayAlign;
        if( 0 < sHeroVerticalDisplayAlignVar )
            RPG::variables[sHeroVerticalDisplayAlignVar] = (int) sHeroVerticalDisplayAlign;
        if( 0 < sMonsterVerticalDisplayAlignVar )
            RPG::variables[sMonsterVerticalDisplayAlignVar] = (int) sMonsterVerticalDisplayAlign;
        if( 0 < sShowHeroATBGaugeVar )
            RPG::variables[sShowHeroATBGaugeVar] = sShowHeroATBGauge;
        if( 0 < sShowMonsterATBGaugeVar )
            RPG::variables[sShowMonsterATBGaugeVar] = sShowMonsterATBGauge;
        if( 0 < sUseSpecialATBVar )
            RPG::variables[sUseSpecialATBVar] = sUseSpecialATB;
        if( 0 < sHeroShowConditionTurnsRemainingVar )
            RPG::variables[sHeroShowConditionTurnsRemainingVar] = sHeroShowConditionTurnsRemaining;
        if( 0 < sMonsterShowConditionTurnsRemainingVar )
            RPG::variables[sMonsterShowConditionTurnsRemainingVar] = sMonsterShowConditionTurnsRemaining;
        if( 0 < sHeroHealthGaugeDurationVar )
            RPG::variables[sHeroHealthGaugeDurationVar] = sHeroHealthGaugeDuration;
        if( 0 < sMonsterHealthGaugeDurationVar )
            RPG::variables[sMonsterHealthGaugeDurationVar] = sMonsterHealthGaugeDuration;
        if( 0 < sHeroManaGaugeDurationVar )
            RPG::variables[sHeroManaGaugeDurationVar] = sHeroManaGaugeDuration;
        if( 0 < sMonsterManaGaugeDurationVar )
            RPG::variables[sMonsterManaGaugeDurationVar] = sMonsterManaGaugeDuration;
        if( 0 < sHeroShowDisplayWhenSelectedVar )
            RPG::variables[sHeroShowDisplayWhenSelectedVar] = sHeroShowDisplayWhenSelected;
        if( 0 < sMonsterShowDisplayWhenSelectedVar )
            RPG::variables[sMonsterShowDisplayWhenSelectedVar] = sMonsterShowDisplayWhenSelected;
        if( 0 < sHeroHealthBarModeVar )
            RPG::variables[sHeroHealthBarModeVar] = (int) sHeroHealthBarMode;
        if( 0 < sMonsterHealthBarModeVar )
            RPG::variables[sMonsterHealthBarModeVar] = (int) sMonsterHealthBarMode;
        if( 0 < sHeroManaBarModeVar )
            RPG::variables[sHeroManaBarModeVar] = (int) sHeroManaBarMode;
        if( 0 < sMonsterManaBarModeVar )
            RPG::variables[sMonsterManaBarModeVar] = (int) sMonsterManaBarMode;
        if( 0 < sHeroHealthChangePauseVar )
            RPG::variables[sHeroHealthChangePauseVar] = sHeroHealthChangePause;
        if( 0 < sMonsterHealthChangePauseVar )
            RPG::variables[sMonsterHealthChangePauseVar] = sMonsterHealthChangePause;
        if( 0 < sHeroManaChangePauseVar )
            RPG::variables[sHeroManaChangePauseVar] = sHeroManaChangePause;
        if( 0 < sMonsterManaChangePauseVar )
            RPG::variables[sMonsterManaChangePauseVar] = sMonsterManaChangePause;
        if( 0 < sHeroHealthChangeDurationVar )
            RPG::variables[sHeroHealthChangeDurationVar] = sHeroHealthChangeDuration;
        if( 0 < sMonsterHealthChangeDurationVar )
            RPG::variables[sMonsterHealthChangeDurationVar] = sMonsterHealthChangeDuration;
        if( 0 < sHeroManaChangeDurationVar )
            RPG::variables[sHeroManaChangeDurationVar] = sHeroManaChangeDuration;
        if( 0 < sMonsterManaChangeDurationVar )
            RPG::variables[sMonsterManaChangeDurationVar] = sMonsterManaChangeDuration;
        if( 0 < sHeroShowHealthNumberVar )
            RPG::variables[sHeroShowHealthNumberVar] = sHeroShowHealthNumber;
        if( 0 < sMonsterShowHealthNumberVar )
            RPG::variables[sMonsterShowHealthNumberVar] = sMonsterShowHealthNumber;
        if( 0 < sHeroShowManaNumberVar )
            RPG::variables[sHeroShowManaNumberVar] = sHeroShowManaNumber;
        if( 0 < sMonsterShowManaNumberVar )
            RPG::variables[sMonsterShowManaNumberVar] = sMonsterShowManaNumber;
        if( 0 < sHeroConditionDurationVar )
            RPG::variables[sHeroConditionDurationVar] = sHeroConditionDuration;
        if( 0 < sMonsterConditionDurationVar )
            RPG::variables[sMonsterConditionDurationVar] = sMonsterConditionDuration;
        if( 0 < sHeroConditionBlinkDurationVar )
            RPG::variables[sHeroConditionBlinkDurationVar] = sHeroConditionBlinkDuration;
        if( 0 < sMonsterConditionBlinkDurationVar )
            RPG::variables[sMonsterConditionBlinkDurationVar] = sMonsterConditionBlinkDuration;
        if( 0 < sHeroConditionBlinkPauseVar )
            RPG::variables[sHeroConditionBlinkPauseVar] = sHeroConditionBlinkPause;
        if( 0 < sMonsterConditionBlinkPauseVar )
            RPG::variables[sMonsterConditionBlinkPauseVar] = sMonsterConditionBlinkPause;
    }

    //! Sets the Battler
    /*!
        SetBattler() sets the value of mBattlerPtr to the given parameter and initializes related
        variables according to the Battler's data.

        \param rBattlerPtr : (RPG::Battler *) Pointer to the Battler which this BattleDisplay will serve
    */
    void SetBattler( RPG::Battler * rBattlerPtr )
    {
        static int i;      // Index variable

        // Initialize variables
        mBattlerPtr = rBattlerPtr;
        mCurHealth = mBattlerPtr->hp;
        mMaxHealth = mBattlerPtr->getMaxHp();
        mAnchorHealth = mCurHealth;
        mHealthGaugeTimer = sHeroHealthGaugeDuration;
        if( mBattlerPtr->isMonster() )
            mHealthGaugeTimer = sMonsterHealthGaugeDuration;
        if( 0 < mHealthGaugeTimer )
            mHealthGaugeTimer = 0;
        mManaGaugeTimer = sHeroManaGaugeDuration;
        if( mBattlerPtr->isMonster() )
            mManaGaugeTimer = sMonsterManaGaugeDuration;
        if( 0 < mManaGaugeTimer )
            mManaGaugeTimer = 0;
        mHealthChangeTimer = 0;
        mManaChangeTimer = 0;
        mCurMana = mBattlerPtr->mp;
        mMaxMana = mBattlerPtr->getMaxMp();
        mAnchorMana = mCurMana;
        mManaChangeTimer = 0;
        mCurATB = mBattlerPtr->atbValue;
        for( i = 0; i < sNumIconConditions; i++ )
        {
            mCurConditionDuration[sIconConditions[i]] = mBattlerPtr->conditions[sIconConditions[i]];
            mConditionTimer[sIconConditions[i]] = 0;
        }
    }

    //! Updates the BattleDisplay
    /*!
        Update() recalculates values based on past and present data and calls Draw() to refresh the
        appearance of the BattleDisplay.
    */
    void Update()
    {
        static int i, j;        // Index variables

        // Decrement timers as necessary
        if( mHealthGaugeTimer > 0 )
        {
            mHealthGaugeTimer--;
        }
        if( mManaGaugeTimer > 0 )
        {
            mManaGaugeTimer--;
        }
        if( mHealthChangeTimer > 0 )
        {
            mHealthChangeTimer--;
        }
        if( mManaChangeTimer > 0 )
        {
            mManaChangeTimer--;
        }
        for( i = 0; i < sNumIconConditions; i++ )
        {
            j = sIconConditions[i];     // Database reference number of the status condition currently being considered
            if( mConditionTimer[j] > 0 )
            {
                mConditionTimer[j]--;
            }
        }
        // Update variables
        if( mCurHealth != mBattlerPtr->hp )
        {   // Health has changed
            // Remember value before most recent change
            mAnchorHealth = mCurHealth;
            // Set the timers
            if( mBattlerPtr->isMonster() )
            {   // Battler is a monster
                mHealthChangeTimer = sMonsterHealthChangePause + sMonsterHealthChangeDuration;
                mHealthGaugeTimer = sMonsterHealthGaugeDuration;
            }
            else
            {   // Battler is a hero
                mHealthChangeTimer = sHeroHealthChangePause + sHeroHealthChangeDuration;
                mHealthGaugeTimer = sHeroHealthGaugeDuration;
            }
            // Set mCurHealth to new value
            mCurHealth = mBattlerPtr->hp;
        }
        if( mMaxHealth != mBattlerPtr->getMaxHp() )
        {   // Maximum health has changed (happens when monster transforms)
            // Reset health-related values
            mCurHealth = mBattlerPtr->hp;
            mMaxHealth = mBattlerPtr->getMaxHp();
            mAnchorHealth = mCurHealth;
        }
        if( mCurMana != mBattlerPtr->mp )
        {   // Mana has changed
            // Remember mana before most recent change
            mAnchorMana = mCurMana;
            // Set the timers
            if( mBattlerPtr->isMonster() )
            {   // Battler is a monster
                mManaChangeTimer = sMonsterManaChangePause + sMonsterManaChangeDuration;
                mManaGaugeTimer = sMonsterManaGaugeDuration;
            }
            else
            {   // Battler is a hero
                mManaChangeTimer = sHeroManaChangePause + sHeroManaChangeDuration;
                mManaGaugeTimer = sHeroManaGaugeDuration;
            }
            // Set mCurMana to new value
            mCurMana = mBattlerPtr->mp;
        }
        if( mMaxMana != mBattlerPtr->getMaxMp() )
        {   // Maximum mana has changed (happens when monster transforms)
            // Reset mana-related values
            mCurMana = mBattlerPtr->mp;
            mMaxMana = mBattlerPtr->getMaxMp();
            mAnchorMana = mCurMana;
        }
        mCurATB = mBattlerPtr->atbValue;
        for( i = 0; i < sNumIconConditions; i++ )
        {
            j = sIconConditions[i];     // Database reference number of the status condition currently being considered
            if( mBattlerPtr->conditions[j] != mCurConditionDuration[j] )
            {   // Number of turns under a condition has changed
                // Set the timer
                if( 0 == mBattlerPtr->conditions[j] )
                {   // Battler is no longer under condition
                    if( mBattlerPtr->isMonster() )
                    {   // Battler is a monster
                        mConditionTimer[j] = sMonsterConditionBlinkDuration;
                    }
                    else
                    {   // Battler is a hero
                        mConditionTimer[j] = sHeroConditionBlinkDuration;
                    }
                }
                else
                {   // Battler is still under condition
                    if( mBattlerPtr->isMonster() )
                    {   // Battler is a monster
                        mConditionTimer[j] = sMonsterConditionDuration;
                    }
                    else
                    {   // Battler is a hero
                        mConditionTimer[j] = sHeroConditionDuration;
                    }
                }
            }
            // Set condition tracker to new value
            mCurConditionDuration[j] = mBattlerPtr->conditions[j];
        }

        // Refresh display
        Draw();
    }

private:

    int mCurHealth;                                     //!< Current health
    int mMaxHealth;                                     //!< Maximum health
    int mAnchorHealth;                                  //!< Health before the latest change
    int mHealthGaugeTimer;                              //!< Timer for tracking how long it should be (in terms of how many times Update() is called) before the health gauge is no longer drawn (-1 indicates always drawn)
    int mHealthChangeTimer;                             //!< Timer for tracking how long it should be (in terms of how many times Update() is called) before the change animation on the health bar should be completed
    int mCurMana;                                       //!< Current mana
    int mMaxMana;                                       //!< Maximum mana
    int mAnchorMana;                                    //!< Mana before the latest change
    int mManaGaugeTimer;                                //!< Timer for tracking how long it should be (in terms of how many times Update() is called) before the mana gauge is no longer drawn (-1 indicates always drawn)
    int mManaChangeTimer;                               //!< Timer for tracking how long it should be (in terms of how many times Update() is called) before the change animation on the mana bar should be completed
    int mCurATB;                                        //!< Current ATB fill value
    std::map<int, int> mCurConditionDuration;           //!< Map of current status condition durations (only represents those conditions which are part of sIconConditions)
    std::map<int, int> mConditionTimer;                 //!< Map of timers for tracking how long it should be (in terms of how many times Update() is called) before the icon for a status condition is no longer drawn (-1 indicates always drawn; only represents those conditions which are part of sIconConditions)

    RPG::Battler * mBattlerPtr;                         //!< Pointer to Battler for which this BattleDisplay is used

    //! Draws the display image
    /*!
        This method draws the display Image based on the relevant data and display rules and puts
        it on the Canvas.
    */
    void Draw()
    {
        static int i, j, k;                     // Index variables
        static int iconCount;                   // Count of how many icons are displayed
        static int turnsRemaining;              // Remaining turns for a status condition
        static RPG::Image * curImage;           // Pointer to image of current element of display being drawn
        static int curX, curY,                  // Current coordinates within the display Image
            curWidth, curHeight;                // Dimensions of current element
        static int visibleWidth, visibleHeight; // Width and height of the display as visible to the player
                                                // (for width, doesn't include condition icons because the blinking makes it too
                                                // complicated and in most cases the gauges will be wider anyway)
        static bool isHighlighted;              // Whether the Battler is currently selected and the configuration says to show the display when selected
        static bool isFast;                     // Whether the Battler currently has any "fast" conditions
        static bool isSlow;                     // Whether the Battler currently has any "slow" conditions
        static float durationProportion;        // Proportion of the bar shrinking animation duration which remains to be done

        // Set up the display Images
        sSystemDisplayPtr->clear();
        if( 0 < sNumIconConditions )
        {
            sIconDisplayPtr->clear();
        }
        visibleWidth = GAUGE_WIDTH;             // Assume the visible width will be at least the width of a gauge for consistency
        visibleHeight = 0;
        // Check whether the Battler is selected and the configuration says to show the display when selected
        isHighlighted = false;
        if( mBattlerPtr->isMonster() && -1 != RPG::getSelectedMonsterIndex() )
        {
            if( sMonsterShowDisplayWhenSelected && mBattlerPtr == RPG::monsters[RPG::getSelectedMonsterIndex()] )
            {
                isHighlighted = true;
            }
        }
        else if( -1 != RPG::battleData->winPartyTarget->getSelected() )
        {
            if( sHeroShowDisplayWhenSelected && mBattlerPtr == RPG::Actor::partyMember( RPG::battleData->winPartyTarget->getSelected() ) )
            {
                isHighlighted = true;
            }
        }
        // Find starting position: lower-left corner of display Image with enough room for one gauge
        curY = sDisplayHeight - GAUGE_HEIGHT;

        // Draw the ATB gauge
        if( ( !mBattlerPtr->isMonster() && sShowHeroATBGauge )
           || ( mBattlerPtr->isMonster() && sShowMonsterATBGauge ) )
        {   // Gauge is set to be shown; proceed with drawing
            curX = 0;
            curImage = sATBGaugePtr;
            curWidth = GAUGE_WIDTH;
            curHeight = GAUGE_HEIGHT;
            sSystemDisplayPtr->draw( curX, curY,                                     // Coordinates in destination Image
                               curImage,                                            // Source Image pointer
                               0, 0,                                                // Coordinates in source Image
                               curWidth, curHeight,                                 // Dimensions in source Image
                               0 );                                                 // Transparency color
            // Draw the ATB bar
            curImage = sATBBarAPtr;
            if( mCurATB >= ATB_MAX )
            {   // Battler has full ATB; switch to bar type B
                curImage = sATBBarBPtr;
            }
            if( sUseSpecialATB )
            {   // Use special "fast" and "slow" ATB bars
                // Determine whether the Battler currently has "fast" conditions, "slow" conditions, or neither/both
                // Special note: multiple doubling/halving statuses do not stack as far as ATB is concerned
                isFast = false;
                isSlow = false;
                for( i = 0; i < (signed int) sFastConditions.size(); i++ )
                {
                    if( mBattlerPtr->conditions[sFastConditions[i]] )
                    {   // Fast condition found
                        isFast = true;
                    }
                }
                for( i = 0; i < (signed int) sSlowConditions.size(); i++ )
                {
                    if( mBattlerPtr->conditions[sSlowConditions[i]] )
                    {   // Slow condition found
                        isSlow = true;
                    }
                }
                if( isFast && !isSlow )
                {   // Only fast conditions
                    curImage = sFastATBBarAPtr;
                    if( ATB_MAX == mCurATB )
                    {   // Battler has full ATB; switch to bar type B
                        curImage = sFastATBBarBPtr;
                    }
                }
                if( isSlow && !isFast )
                {   // Only slow conditions
                    curImage = sSlowATBBarAPtr;
                    if( ATB_MAX == mCurATB )
                    {   // Battler has full ATB; switch to bar type B
                        curImage = sSlowATBBarBPtr;
                    }
                }
            }
            curX += BAR_X_OFFSET;
            curWidth = BAR_WIDTH * mCurATB / ATB_MAX;
            if( curWidth < 0 )
            {   // Calculated bar width is less than 0 (shouldn't happen, but programming defensively)
                curWidth = 0;
            }
            if( curWidth > BAR_WIDTH )
            {   // Calculated bar width is greater than expected max (don't know of any cases where this could happen, but programming defensively)
                // Set curWidth to maximum bar width
                curWidth = BAR_WIDTH;
            }
            curHeight = BAR_HEIGHT;
            sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                               curImage,                                            // Source Image pointer
                               0, 0,                                                // Coordinates in source Image
                               curWidth, curHeight,                                 // Dimensions in source Image
                               0 );                                                 // Transparency color
            // Move up within display by the amount designated for the offset between gauges
            if( mBattlerPtr->isMonster() )
            {
                curY -= sMonsterATBGaugeYOffset;
            }
            else
            {
                curY -= sHeroATBGaugeYOffset;
            }
            // Record current visible width
            if( visibleWidth < GAUGE_WIDTH )
            {
                visibleWidth = GAUGE_WIDTH;
            }
        }

        // Draw the mana gauge
        if( 0 != mManaGaugeTimer        // Gauge timer has not reached 0
            || (
                isHighlighted            // Battler is highlighted
                && (
                    ( !mBattlerPtr->isMonster() && 0 != sHeroManaGaugeDuration )    // Battler is a hero and gauge is set to be shown for heroes
                    || ( mBattlerPtr->isMonster() && 0 != sMonsterManaGaugeDuration )  // Battler is a monster and gauge is set to be shown for monsters
                )
            ) )
        {   // Proceed with drawing
            curX = 0;                   // Move to X position for a gauge
            curImage = sManaGaugePtr;
            curWidth = GAUGE_WIDTH;
            curHeight = GAUGE_HEIGHT;
            sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                               curImage,                                            // Source Image pointer
                               0, 0,                                                // Coordinates in source Image
                               curWidth, curHeight,                                 // Dimensions in source Image
                               0 );                                                 // Transparency color
            // Draw the mana bar
            curX += BAR_X_OFFSET;
            curHeight = BAR_HEIGHT;
            if( 0 == mBattlerPtr->getMaxMp() )
            {   // Battler does not have mana at all
                // Do nothing, mana gauge will display as empty
            }
            else if( ( !mBattlerPtr->isMonster() && REGULAR == sHeroManaBarMode )
               || ( mBattlerPtr->isMonster() && REGULAR == sMonsterManaBarMode ) )
            {   // Use regular RM2K3 mode for bar
                curImage = sManaBarAPtr;
                if( mCurMana >= mBattlerPtr->getMaxMp() )
                {   // Battler has full mana; switch to bar type B
                    curImage = sManaBarBPtr;
                }
                if( curWidth < 0 )
                {   // Calculated bar width is less than 0 (shouldn't happen, but programming defensively)
                    curWidth = 0;
                }
                curWidth = BAR_WIDTH * mCurMana / mBattlerPtr->getMaxMp();
                if( curWidth < 0 )
                {   // Calculated bar width is less than 0 (shouldn't happen, but programming defensively)
                    curWidth = 0;
                }
                if( curWidth > BAR_WIDTH )
                {   // Calculated bar width is greater than expected max (can happen with transformed monsters)
                    // Set curWidth to maximum bar width
                    curWidth = BAR_WIDTH;
                }
                sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                                   curImage,                                            // Source Image pointer
                                   0, 0,                                                // Coordinates in source Image
                                   curWidth, curHeight,                                 // Dimensions in source Image
                                   0 );                                                 // Transparency color
            }
            else if( ( !mBattlerPtr->isMonster() && ENHANCED == sHeroManaBarMode )
               || ( mBattlerPtr->isMonster() && ENHANCED == sMonsterManaBarMode ) )
            {   // Use enhanced mode for bar
                // Calculate proportion of the shrinking duration which remains for later use
                if( mBattlerPtr->isMonster() )
                {   // Battler is a monster
                    durationProportion = (float) mManaChangeTimer / sMonsterManaChangeDuration;
                }
                else
                {   // Battler is a hero
                    durationProportion = (float) mManaChangeTimer / sHeroManaChangeDuration;
                }
                // First draw the back (A) bar
                curImage = sManaBarAPtr;
                if( 0 == mManaChangeTimer || mCurMana >= mAnchorMana )
                {   // Change portion of bar is non-existent or shrinking upward
                    // Use current value for the back bar
                    curWidth = BAR_WIDTH * mCurMana / mBattlerPtr->getMaxMp();
                }
                else
                {   // Change portion of bar is shrinking downward
                    if( ( !mBattlerPtr->isMonster() && mManaChangeTimer > sHeroManaChangeDuration )
                       || ( mBattlerPtr->isMonster() && mManaChangeTimer > sMonsterManaChangeDuration ) )
                    {   // Currently in pause part of bar change animation
                        // Use length out to anchor value
                        curWidth = BAR_WIDTH * mAnchorMana  / mBattlerPtr->getMaxMp();
                    }
                    else
                    {   // Currently in shrinking part of bar change animation
                        // Use partial length according to proportion of shrinking duration remaining
                        curWidth = BAR_WIDTH * ( mCurMana + ( mAnchorMana - mCurMana ) * durationProportion ) / mBattlerPtr->getMaxMp();
                    }
                }
                if( curWidth < 0 )
                {   // Calculated bar width is less than 0 (shouldn't happen, but programming defensively)
                    curWidth = 0;
                }
                if( curWidth > BAR_WIDTH )
                {   // Calculated bar width is greater than expected max (can happen with transformed monsters)
                    // Set curWidth to maximum bar width
                    curWidth = BAR_WIDTH;
                }
                sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                                   curImage,                                            // Source Image pointer
                                   0, 0,                                                // Coordinates in source Image
                                   curWidth, curHeight,                                 // Dimensions in source Image
                                   0 );                                                 // Transparency color
                // Next draw the front (B) bar over top of the back one
                curImage = sManaBarBPtr;
                if( 0 == mManaChangeTimer || mCurMana <= mAnchorMana )
                {   // Change portion of bar is non-existent or shrinking downward
                    // Use current value for the front bar
                    curWidth = BAR_WIDTH * mCurMana / mBattlerPtr->getMaxMp();
                }
                else
                {   // Change portion of bar is shrinking upward
                    // Use anchor value for the front bar
                    if( ( !mBattlerPtr->isMonster() && mManaChangeTimer > sHeroManaChangeDuration )
                       || ( mBattlerPtr->isMonster() && mManaChangeTimer > sMonsterManaChangeDuration ) )
                    {   // Currently in pause part of bar change animation
                        // Use length out to anchor value
                        curWidth = BAR_WIDTH * mAnchorMana / mBattlerPtr->getMaxMp();
                    }
                    else
                    {   // Currently in shrinking part of bar change animation
                        // Use partial length according to proportion of shrinking duration remaining
                        curWidth = BAR_WIDTH * ( mCurMana - ( mCurMana - mAnchorMana ) * durationProportion ) / mBattlerPtr->getMaxMp();
                    }
                }
                if( curWidth < 0 )
                {   // Calculated bar width is less than 0 (shouldn't happen, but programming defensively)
                    curWidth = 0;
                }
                if( curWidth > BAR_WIDTH )
                {   // Calculated bar width is greater than expected max (can happen with transformed monsters)
                    // Set curWidth to maximum bar width
                    curWidth = BAR_WIDTH;
                }
                sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                                   curImage,                                            // Source Image pointer
                                   0, 0,                                                // Coordinates in source Image
                                   curWidth, curHeight,                                 // Dimensions in source Image
                                   0 );                                                 // Transparency color
            }
            // Record current visible width
            if( visibleWidth < GAUGE_WIDTH )
            {
                visibleWidth = GAUGE_WIDTH;
            }
            // Draw exact number if desired
            if( ( !mBattlerPtr->isMonster() && sHeroShowManaNumber )
               || ( mBattlerPtr->isMonster() && sMonsterShowManaNumber ) )
            {   // Exact number desired
                curX = sHeroManaNumberXOffset;
                if( mBattlerPtr->isMonster() )
                {
                    curX = sMonsterManaNumberXOffset;
                }
                curWidth = DIGIT_WIDTH;
                curHeight = DIGIT_HEIGHT;
                for( i = 1000; i > 0; i /= 10 )        // Loop though all possible powers of ten from greatest to least
                {
                    if( mBattlerPtr->mp >= i || ( 1 == i && 0 == mBattlerPtr->mp ) )
                    {   // Number is at least the currently considered power of ten OR number is 0 and current power of ten equals 1
                        curImage = sDigitPtr[( mBattlerPtr->mp / i ) % 10];
                        sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                                           curImage,                                            // Source Image pointer
                                           0, 0,                                                // Coordinates in source Image
                                           curWidth, curHeight,                                 // Dimensions in source Image
                                           0 );                                                 // Transparency color
                        curX += DIGIT_WIDTH;
                    }
                }
                // Record current visible width
                if( visibleWidth < curX )
                {
                    visibleWidth = curX;
                }
            }
            // Move up within display by the amount designated for the offset between gauges
            if( mBattlerPtr->isMonster() )
            {
                curY -= sMonsterManaGaugeYOffset;
            }
            else
            {
                curY -= sHeroManaGaugeYOffset;
            }
        }

        // Draw the health gauge
        if( 0 != mHealthGaugeTimer      // Gauge timer has not reached 0
            || (
                isHighlighted            // Battler is highlighted
                && (
                    ( !mBattlerPtr->isMonster() && 0 != sHeroHealthGaugeDuration )      // Battler is a hero and gauge is set to be shown for heroes
                    || ( mBattlerPtr->isMonster() && 0 != sMonsterHealthGaugeDuration )    // Battler is a monster and gauge is set to be shown for monsters
                )
            ) )
        {   // Proceed with drawing
            curX = 0;                   // Move to X position for a gauge
            curImage = sHealthGaugePtr;
            curWidth = GAUGE_WIDTH;
            curHeight = GAUGE_HEIGHT;
            sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                               curImage,                                            // Source Image pointer
                               0, 0,                                                // Coordinates in source Image
                               curWidth, curHeight,                                 // Dimensions in source Image
                               0 );                                                 // Transparency color
            // Draw the health bar
            curX += BAR_X_OFFSET;
            curHeight = BAR_HEIGHT;
            if( 0 == mBattlerPtr->getMaxHp() )
            {   // Battler does not have health at all (should not happen in most cases, but programming defensively)
                // Do nothing, health gauge will display as empty
            }
            else if( ( !mBattlerPtr->isMonster() && REGULAR == sHeroHealthBarMode )
               || ( mBattlerPtr->isMonster() && REGULAR == sMonsterHealthBarMode ) )
            {   // Use regular RM2K3 mode for bar
                curImage = sHealthBarAPtr;
                if( mCurHealth >= mBattlerPtr->getMaxHp() )
                {   // Battler has full health; switch to bar type B
                    curImage = sHealthBarBPtr;
                }
                curWidth = BAR_WIDTH * mCurHealth / mBattlerPtr->getMaxHp();
                if( curWidth < 0 )
                {   // Calculated bar width is less than 0 (shouldn't happen, but programming defensively)
                    curWidth = 0;
                }
                if( curWidth > BAR_WIDTH )
                {   // Calculated bar width is greater than expected max (can happen with transformed monsters)
                    // Set curWidth to maximum bar width
                    curWidth = BAR_WIDTH;
                }
                sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                                   curImage,                                            // Source Image pointer
                                   0, 0,                                                // Coordinates in source Image
                                   curWidth, curHeight,                                 // Dimensions in source Image
                                   0 );                                                 // Transparency color
            }
            else if( ( !mBattlerPtr->isMonster() && ENHANCED == sHeroHealthBarMode )
               || ( mBattlerPtr->isMonster() && ENHANCED == sMonsterHealthBarMode ) )
            {   // Use enhanced mode for bar
                // Calculate proportion of the shrinking duration which remains for later use
                if( mBattlerPtr->isMonster() )
                {   // Battler is a monster
                    durationProportion = (float) mHealthChangeTimer / sMonsterHealthChangeDuration;
                }
                else
                {   // Battler is a hero
                    durationProportion = (float) mHealthChangeTimer / sHeroHealthChangeDuration;
                }
                // First draw the back (A) bar
                curImage = sHealthBarAPtr;
                if( 0 == mHealthChangeTimer || mCurHealth >= mAnchorHealth )
                {   // Change portion of bar is non-existent or shrinking upward
                    // Use current value for the back bar
                    curWidth = BAR_WIDTH * mCurHealth / mBattlerPtr->getMaxHp();
                }
                else
                {   // Change portion of bar is shrinking downward
                    if( ( !mBattlerPtr->isMonster() && mHealthChangeTimer > sHeroHealthChangeDuration )
                       || ( mBattlerPtr->isMonster() && mHealthChangeTimer > sMonsterHealthChangeDuration ) )
                    {   // Currently in pause part of bar change animation
                        // Use length out to anchor value
                        curWidth = BAR_WIDTH * mAnchorHealth  / mBattlerPtr->getMaxHp();
                    }
                    else
                    {   // Currently in shrinking part of bar change animation
                        // Use partial length according to proportion of shrinking duration remaining
                        curWidth = BAR_WIDTH * ( mCurHealth + ( mAnchorHealth - mCurHealth ) * durationProportion ) / mBattlerPtr->getMaxHp();
                    }
                }
                if( curWidth < 0 )
                {   // Calculated bar width is less than 0 (shouldn't happen, but programming defensively)
                    curWidth = 0;
                }
                if( curWidth > BAR_WIDTH )
                {   // Calculated bar width is greater than expected max (can happen with transformed monsters)
                    // Set curWidth to maximum bar width
                    curWidth = BAR_WIDTH;
                }
                sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                                   curImage,                                            // Source Image pointer
                                   0, 0,                                                // Coordinates in source Image
                                   curWidth, curHeight,                                 // Dimensions in source Image
                                   0 );                                                 // Transparency color
                // Next draw the front (B) bar over top of the back one
                curImage = sHealthBarBPtr;
                if( 0 == mHealthChangeTimer || mCurHealth <= mAnchorHealth )
                {   // Change portion of bar is non-existent or shrinking downward
                    // Use current value for the front bar
                    curWidth = BAR_WIDTH * mCurHealth / mBattlerPtr->getMaxHp();
                }
                else
                {   // Change portion of bar is shrinking upward
                    // Use anchor value for the front bar
                    if( ( !mBattlerPtr->isMonster() && mHealthChangeTimer > sHeroHealthChangeDuration )
                       || ( mBattlerPtr->isMonster() && mHealthChangeTimer > sMonsterHealthChangeDuration ) )
                    {   // Currently in pause part of bar change animation
                        // Use length out to anchor value
                        curWidth = BAR_WIDTH * mAnchorHealth  / mBattlerPtr->getMaxHp();
                    }
                    else
                    {   // Currently in shrinking part of bar change animation
                        // Use partial length according to proportion of shrinking duration remaining
                        curWidth = BAR_WIDTH * ( mCurHealth - ( mCurHealth - mAnchorHealth ) * durationProportion ) / mBattlerPtr->getMaxHp();
                    }
                }
                if( curWidth < 0 )
                {   // Calculated bar width is less than 0 (shouldn't happen, but programming defensively)
                    curWidth = 0;
                }
                if( curWidth > BAR_WIDTH )
                {   // Calculated bar width is greater than expected max (can happen with transformed monsters)
                    // Set curWidth to maximum bar width
                    curWidth = BAR_WIDTH;
                }
                sSystemDisplayPtr->draw( curX, curY,                                     // Coordinates in destination Image
                                   curImage,                                            // Source Image pointer
                                   0, 0,                                                // Coordinates in source Image
                                   curWidth, curHeight,                                 // Dimensions in source Image
                                   0 );                                                 // Transparency color
            }
            // Record current visible width
            if( visibleWidth < GAUGE_WIDTH )
            {
                visibleWidth = GAUGE_WIDTH;
            }
            // Draw exact number if desired
            if( ( !mBattlerPtr->isMonster() && sHeroShowHealthNumber )
               || ( mBattlerPtr->isMonster() && sMonsterShowHealthNumber ) )
            {   // Exact number desired
                curX = sHeroHealthNumberXOffset;
                if( mBattlerPtr->isMonster() )
                {
                    curX = sMonsterHealthNumberXOffset;
                }
                curWidth = DIGIT_WIDTH;
                curHeight = DIGIT_HEIGHT;
                for( i = 10000; i > 0; i /= 10 )        // Loop though all possible powers of ten from greatest to least
                {
                    if( mBattlerPtr->hp >= i || ( 1 == i && 0 == mBattlerPtr->hp ) )
                    {   // Number is at least the currently considered power of ten OR number is 0 and current power of ten equals 1
                        curImage = sDigitPtr[( mBattlerPtr->hp / i ) % 10];
                        sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                                           curImage,                                            // Source Image pointer
                                           0, 0,                                                // Coordinates in source Image
                                           curWidth, curHeight,                                 // Dimensions in source Image
                                           0 );                                                 // Transparency color
                        curX += DIGIT_WIDTH;
                    }
                }
                // Record current visible width
                if( visibleWidth < curX )
                {
                    visibleWidth = curX;
                }
            }
            // Move up within display by the amount designated for the offset between gauges
            if( mBattlerPtr->isMonster() )
            {
                curY -= sMonsterHealthGaugeYOffset;
            }
            else
            {
                curY -= sHeroHealthGaugeYOffset;
            }
        }

        // Draw any appropriate status condition icons
        curY += GAUGE_HEIGHT;       // Move downward within the display so that curY is aligned with the top of the previously drawn gauge
        iconCount = 0;
        for( i = 0; i < sNumIconConditions; i++ )
        {
            // Check whether there is room for another icon (maximum has not been reached)
            if( ( !mBattlerPtr->isMonster() && iconCount < sHeroMaxConditionDisplay )
                || ( mBattlerPtr->isMonster() && iconCount < sMonsterMaxConditionDisplay ) )
            {
                j = sIconConditions[i];     // Database reference number of the status condition currently being considered
                // Determine whether the Battler has a status condition which changed within the condition show duration
                if( mConditionTimer[j] != 0                                     // Condition changed within show duration
                    || ( isHighlighted && 0 != mBattlerPtr->conditions[j] ) )   // Battler is highlighted and has the condition
                {   // Proceed with drawing
                    // Move up within display by the amount designated for the offset between condition icons
                    curY -= sConditionYOffset;
                    // Determine whether to actually draw icon
                    if( mBattlerPtr->conditions[j]                                                     // Still under status condition
                        || ( !mBattlerPtr->isMonster() && ( mConditionTimer[j] / sHeroConditionBlinkPause ) % 2 )       // Currently in "show" part of blinking animation
                        || ( mBattlerPtr->isMonster() && ( mConditionTimer[j] / sMonsterConditionBlinkPause ) % 2 ) )     // Currently in "show" part of blinking animation
                    {   // Do draw the icon
                        curX = 0;
                        curImage = sConditionIconPtr[j];
                        curWidth = sConditionIconWidth;
                        curHeight = sConditionIconHeight;
                        sIconDisplayPtr->draw( curX, curY,                                   // Coordinates in destination Image
                                           curImage,                                        // Source Image pointer
                                           0, 0,                                            // Coordinates in source Image
                                           curWidth, curHeight,                             // Dimensions in source Image
                                           0 );                                             // Transparency color
                        if
                        (
                            0 < RPG::conditions[j]->recoveryTurns          // The status condition naturally abates after a certain number of turns
                            &&
                            (
                                ( !mBattlerPtr->isMonster() && sHeroShowConditionTurnsRemaining )         // Battler is a hero and the plugin is configured to show max turns for heroes
                                || ( mBattlerPtr->isMonster() && sMonsterShowConditionTurnsRemaining )    // Battler is a monster and the plugin is configured to show max turns for monsters
                            )
                        )
                        {
                            // Draw digits next to the status condition icon
                            curX += sConditionIconWidth;
                            curWidth = DIGIT_WIDTH;
                            curHeight = DIGIT_HEIGHT;
                            // For some odd reason, RPG Maker 2003 doesn't seem to do turn-based
                            // condition recovery until the number of turns a battler has been
                            // inflicted with a condition exceeds the listed recovery turns by 2. I
                            // suspect this is because it discounts the turn of the battler that
                            // inflicted the condition in the first place and also wants the
                            // recovery to take place strictly /after/ the recovery turns have
                            // passed.
                            turnsRemaining = RPG::conditions[j]->recoveryTurns - mBattlerPtr->conditions[j] + 2;
                            if( turnsRemaining < 0 ) turnsRemaining = 0;      // Make sure turnsRemaining does not go into negative values
                            // Special case: if Battler is listed as having been afflicted for 0 turns,
                            // it indicates the condition just abated; set turnsRemaining to 0
                            if( 0 == mBattlerPtr->conditions[j] ) turnsRemaining = 0;
                            for( k = 1000; k > 0; k /= 10 )        // Loop though all possible powers of ten from greatest to least
                            {
                                if( turnsRemaining >= k || ( 1 == k && 0 == turnsRemaining ) )
                                {   // Number is at least the currently considered power of ten OR number is 0 and current power of ten equals 1
                                    curImage = sDigitPtr[( turnsRemaining / k ) % 10];
                                    sSystemDisplayPtr->draw( curX, curY,                                    // Coordinates in destination Image
                                                       curImage,                                            // Source Image pointer
                                                       0, 0,                                                // Coordinates in source Image
                                                       curWidth, curHeight,                                 // Dimensions in source Image
                                                       0 );                                                 // Transparency color
                                    curX += DIGIT_WIDTH;
                                }
                            }
                            // Record current visible width
                            if( visibleWidth < curX )
                            {
                                visibleWidth = curX;
                            }
                        }
                    }
                    iconCount++;
                }
            }
        }

        // Draw the battle display onto the canvas
        // Find the desired coordinates for the display relative to the center of the battler sprite
        curWidth = sHeroSpriteWidth;
        curHeight = sHeroSpriteHeight;
        if( mBattlerPtr->isMonster() )
        {
            curWidth = ( (RPG::Monster *) mBattlerPtr )->image->width;
            curHeight = ( (RPG::Monster *) mBattlerPtr )->image->height;
        }
        if( ( !mBattlerPtr->isMonster() && RIGHT_ALIGN_TO_LEFT == sHeroHorizontalDisplayAlign )
            || ( mBattlerPtr->isMonster() && RIGHT_ALIGN_TO_LEFT == sMonsterHorizontalDisplayAlign ) )
        {   // Align right side of display to left side of battler sprite
            curX = -curWidth / 2 - visibleWidth;
        }
        else if( ( !mBattlerPtr->isMonster() && CENTER_ON_LEFT == sHeroHorizontalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && CENTER_ON_LEFT == sMonsterHorizontalDisplayAlign ) )
        {   // Center display on left side of battler sprite
            curX = ( -curWidth - visibleWidth ) / 2;
        }
        else if( ( !mBattlerPtr->isMonster() && LEFT_ALIGN_TO_LEFT == sHeroHorizontalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && LEFT_ALIGN_TO_LEFT == sMonsterHorizontalDisplayAlign ) )
        {   // Align left side of display to left side of battler sprite
            curX = -curWidth / 2;
        }
        else if( ( !mBattlerPtr->isMonster() && RIGHT_ALIGN_TO_CENTER == sHeroHorizontalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && RIGHT_ALIGN_TO_CENTER == sMonsterHorizontalDisplayAlign ) )
        {   // Align right side of display to center of battler sprite
            curX = -visibleWidth;
        }
        else if( ( !mBattlerPtr->isMonster() && CENTER_ON_CENTER == sHeroHorizontalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && CENTER_ON_CENTER == sMonsterHorizontalDisplayAlign ) )
        {   // Center display on center of battler sprite
            curX = -visibleWidth / 2;
        }
        else if( ( !mBattlerPtr->isMonster() && LEFT_ALIGN_TO_CENTER == sHeroHorizontalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && LEFT_ALIGN_TO_CENTER == sMonsterHorizontalDisplayAlign ) )
        {   // Align left side of display to center of battler sprite
            curX = 0;
        }
        else if( ( !mBattlerPtr->isMonster() && RIGHT_ALIGN_TO_RIGHT == sHeroHorizontalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && RIGHT_ALIGN_TO_RIGHT == sMonsterHorizontalDisplayAlign ) )
        {   // Align right side of display to right side of battler sprite
            curX = curWidth / 2 - visibleWidth;
        }
        else if( ( !mBattlerPtr->isMonster() && CENTER_ON_RIGHT == sHeroHorizontalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && CENTER_ON_RIGHT == sMonsterHorizontalDisplayAlign ) )
        {   // Center display on left side of battler sprite
            curX = ( curWidth - visibleWidth ) / 2;
        }
        else if( ( !mBattlerPtr->isMonster() && RIGHT_ALIGN_TO_LEFT == sHeroHorizontalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && RIGHT_ALIGN_TO_LEFT == sMonsterHorizontalDisplayAlign ) )
        {   // Align left side of display to right side of battler sprite
            curX = curWidth / 2;
        }
        // Note that curY should still hold the Y position within the display of the last drawn
        // display element; this is useful as it gives the amount of empty space at the top of the
        // display. Also, the complement of this to the maximum possible display height should be
        // equal to the visible height of the display to the player.
        i = curY;                       // Remember the height of the empty space
        visibleHeight = sDisplayHeight - curY;      // Get the height of the visible space
        if( ( !mBattlerPtr->isMonster() && TOP_ALIGN_TO_BOTTOM == sHeroVerticalDisplayAlign )
            || ( mBattlerPtr->isMonster() && TOP_ALIGN_TO_BOTTOM == sMonsterVerticalDisplayAlign ) )
        {   // Align top side of display to bottom side of battler sprite
            curY = curHeight / 2;
        }
        else if( ( !mBattlerPtr->isMonster() && MIDDLE_ON_BOTTOM == sHeroVerticalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && MIDDLE_ON_BOTTOM == sMonsterVerticalDisplayAlign ) )
        {   // Middle display on bottom side of battler sprite
            curY = ( curHeight - visibleHeight ) / 2;
        }
        else if( ( !mBattlerPtr->isMonster() && BOTTOM_ALIGN_TO_BOTTOM == sHeroVerticalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && BOTTOM_ALIGN_TO_BOTTOM == sMonsterVerticalDisplayAlign ) )
        {   // Align bottom side of display to bottom side of battler sprite
            curY = curHeight / 2 - visibleHeight;
        }
        else if( ( !mBattlerPtr->isMonster() && TOP_ALIGN_TO_MIDDLE == sHeroVerticalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && TOP_ALIGN_TO_MIDDLE == sMonsterVerticalDisplayAlign ) )
        {   // Align top side of display to middle of battler sprite
            curY = 0;
        }
        else if( ( !mBattlerPtr->isMonster() && MIDDLE_ON_MIDDLE == sHeroVerticalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && MIDDLE_ON_MIDDLE == sMonsterVerticalDisplayAlign ) )
        {   // Middle display on middle of battler sprite
            curY = -visibleHeight / 2;
        }
        else if( ( !mBattlerPtr->isMonster() && BOTTOM_ALIGN_TO_MIDDLE == sHeroVerticalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && BOTTOM_ALIGN_TO_MIDDLE == sMonsterVerticalDisplayAlign ) )
        {   // Align bottom side of display to middle of battler sprite
            curY = -visibleHeight;
        }
        else if( ( !mBattlerPtr->isMonster() && TOP_ALIGN_TO_TOP == sHeroVerticalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && TOP_ALIGN_TO_TOP == sMonsterVerticalDisplayAlign ) )
        {   // Align top side of display to top side of battler sprite
            curY = -curHeight / 2;
        }
        else if( ( !mBattlerPtr->isMonster() && MIDDLE_ON_TOP == sHeroVerticalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && MIDDLE_ON_TOP == sMonsterVerticalDisplayAlign ) )
        {   // Middle display on bottom side of battler sprite
            curY = ( -curHeight - visibleHeight ) / 2;
        }
        else if( ( !mBattlerPtr->isMonster() && BOTTOM_ALIGN_TO_TOP == sHeroVerticalDisplayAlign )
                 || ( mBattlerPtr->isMonster() && BOTTOM_ALIGN_TO_TOP == sMonsterVerticalDisplayAlign ) )
        {   // Align bottom side of display to top side of battler sprite
            curY = -curHeight / 2 - visibleHeight;
        }
        // Subtract the height of the empty space in the display
        curY -= i;
        // Add the coordinates of the center of the battler sprite to get the battle display's
        // canvas position
        curX += mBattlerPtr->x;
        curY += mBattlerPtr->y;
        // Draw the display images
        RPG::screen->canvas->draw( curX, curY, sSystemDisplayPtr );
        if( 0 < iconCount )
        {
            RPG::screen->canvas->draw( curX, curY, sIconDisplayPtr );
        }
    }

};

bool BattleDisplay::sShowHeroATBGauge = true;
bool BattleDisplay::sShowMonsterATBGauge = true;
bool BattleDisplay::sUseSpecialATB = false;
bool BattleDisplay::sHeroShowHealthNumber = true;
bool BattleDisplay::sMonsterShowHealthNumber = true;
bool BattleDisplay::sHeroShowManaNumber = true;
bool BattleDisplay::sMonsterShowManaNumber = true;
bool BattleDisplay::sHeroShowDisplayWhenSelected = true;
bool BattleDisplay::sMonsterShowDisplayWhenSelected = true;
bool BattleDisplay::sHeroShowConditionTurnsRemaining = true;
bool BattleDisplay::sMonsterShowConditionTurnsRemaining = true;
BattleDisplay::BarMode BattleDisplay::sHeroHealthBarMode = BattleDisplay::ENHANCED;
BattleDisplay::BarMode BattleDisplay::sMonsterHealthBarMode = BattleDisplay::ENHANCED;
BattleDisplay::BarMode BattleDisplay::sHeroManaBarMode = BattleDisplay::ENHANCED;
BattleDisplay::BarMode BattleDisplay::sMonsterManaBarMode = BattleDisplay::ENHANCED;
BattleDisplay::DisplayHorizontalAlign BattleDisplay::sHeroHorizontalDisplayAlign = BattleDisplay::CENTER_ON_CENTER;
BattleDisplay::DisplayHorizontalAlign BattleDisplay::sMonsterHorizontalDisplayAlign = BattleDisplay::CENTER_ON_CENTER;
BattleDisplay::DisplayVerticalAlign BattleDisplay::sHeroVerticalDisplayAlign = BattleDisplay::BOTTOM_ALIGN_TO_TOP;
BattleDisplay::DisplayVerticalAlign BattleDisplay::sMonsterVerticalDisplayAlign = BattleDisplay::BOTTOM_ALIGN_TO_TOP;
int BattleDisplay::sDisplayWidth = BattleDisplay::DISPLAY_WIDTH;
int BattleDisplay::sDisplayHeight = BattleDisplay::DISPLAY_HEIGHT;
int BattleDisplay::sHeroSpriteWidth = BattleDisplay::HERO_SPRITE_WIDTH;
int BattleDisplay::sHeroSpriteHeight = BattleDisplay::HERO_SPRITE_HEIGHT;
int BattleDisplay::sHeroHealthGaugeDuration = BattleDisplay::HERO_HEALTH_GAUGE_DURATION;
int BattleDisplay::sMonsterHealthGaugeDuration = BattleDisplay::MONSTER_HEALTH_GAUGE_DURATION;
int BattleDisplay::sHeroManaGaugeDuration = BattleDisplay::HERO_MANA_GAUGE_DURATION;
int BattleDisplay::sMonsterManaGaugeDuration = BattleDisplay::MONSTER_MANA_GAUGE_DURATION;
int BattleDisplay::sHeroHealthChangePause = BattleDisplay::HERO_HEALTH_CHANGE_PAUSE;
int BattleDisplay::sMonsterHealthChangePause = BattleDisplay::MONSTER_HEALTH_CHANGE_PAUSE;
int BattleDisplay::sHeroManaChangePause = BattleDisplay::HERO_MANA_CHANGE_PAUSE;
int BattleDisplay::sMonsterManaChangePause = BattleDisplay::MONSTER_MANA_CHANGE_PAUSE;
int BattleDisplay::sHeroHealthChangeDuration = BattleDisplay::HERO_HEALTH_CHANGE_DURATION;
int BattleDisplay::sMonsterHealthChangeDuration = BattleDisplay::MONSTER_HEALTH_CHANGE_DURATION;
int BattleDisplay::sHeroManaChangeDuration = BattleDisplay::HERO_MANA_CHANGE_DURATION;
int BattleDisplay::sMonsterManaChangeDuration = BattleDisplay::MONSTER_MANA_CHANGE_DURATION;
int BattleDisplay::sConditionIconWidth = BattleDisplay::CONDITION_ICON_WIDTH;
int BattleDisplay::sConditionIconHeight = BattleDisplay::CONDITION_ICON_HEIGHT;
int BattleDisplay::sHeroMaxConditionDisplay = BattleDisplay::HERO_MAX_CONDITION_DISPLAY;
int BattleDisplay::sMonsterMaxConditionDisplay = BattleDisplay::MONSTER_MAX_CONDITION_DISPLAY;
int BattleDisplay::sHeroConditionDuration = BattleDisplay::HERO_CONDITION_DURATION;
int BattleDisplay::sMonsterConditionDuration = BattleDisplay::MONSTER_CONDITION_DURATION;
int BattleDisplay::sHeroConditionBlinkDuration = BattleDisplay::HERO_CONDITION_BLINK_DURATION;
int BattleDisplay::sMonsterConditionBlinkDuration = BattleDisplay::MONSTER_CONDITION_BLINK_DURATION;
int BattleDisplay::sHeroConditionBlinkPause = BattleDisplay::HERO_CONDITION_BLINK_PAUSE;
int BattleDisplay::sMonsterConditionBlinkPause = BattleDisplay::MONSTER_CONDITION_BLINK_PAUSE;

RPG::Image * BattleDisplay::sHealthGaugePtr = RPG::Image::create( BattleDisplay::GAUGE_WIDTH, BattleDisplay::GAUGE_HEIGHT );
RPG::Image * BattleDisplay::sManaGaugePtr = RPG::Image::create( BattleDisplay::GAUGE_WIDTH, BattleDisplay::GAUGE_HEIGHT );
RPG::Image * BattleDisplay::sATBGaugePtr = RPG::Image::create( BattleDisplay::GAUGE_WIDTH, BattleDisplay::GAUGE_HEIGHT );
RPG::Image * BattleDisplay::sHealthBarAPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sManaBarAPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sATBBarAPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sHealthBarBPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sManaBarBPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sATBBarBPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sFastATBBarAPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sFastATBBarBPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sSlowATBBarAPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sSlowATBBarBPtr = RPG::Image::create( BattleDisplay::BAR_WIDTH, BattleDisplay::BAR_HEIGHT );
RPG::Image * BattleDisplay::sDigitPtr[BattleDisplay::NUM_DIGITS] = {
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ),
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ),
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ),
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ),
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ),
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ),
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ),
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ),
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ),
    RPG::Image::create( BattleDisplay::DIGIT_WIDTH, BattleDisplay::DIGIT_HEIGHT ) };
std::map<int, RPG::Image *> BattleDisplay::sConditionIconPtr;
RPG::Image * BattleDisplay::sSystemDisplayPtr;
RPG::Image * BattleDisplay::sIconDisplayPtr;

std::vector<int> BattleDisplay::sFastConditions;
std::vector<int> BattleDisplay::sSlowConditions;
std::vector<int> BattleDisplay::sIconConditions;
int BattleDisplay::sNumIconConditions = 0;

int BattleDisplay::sHeroHealthNumberXOffset = BattleDisplay::HERO_HEALTH_NUMBER_X_OFFSET;
int BattleDisplay::sMonsterHealthNumberXOffset = BattleDisplay::MONSTER_HEALTH_NUMBER_X_OFFSET;
int BattleDisplay::sHeroManaNumberXOffset = BattleDisplay::HERO_MANA_NUMBER_X_OFFSET;
int BattleDisplay::sMonsterManaNumberXOffset = BattleDisplay::MONSTER_MANA_NUMBER_X_OFFSET;
int BattleDisplay::sHeroHealthGaugeYOffset = BattleDisplay::HERO_HEALTH_GAUGE_Y_OFFSET;
int BattleDisplay::sMonsterHealthGaugeYOffset = BattleDisplay::MONSTER_HEALTH_GAUGE_Y_OFFSET;
int BattleDisplay::sHeroManaGaugeYOffset = BattleDisplay::HERO_MANA_GAUGE_Y_OFFSET;
int BattleDisplay::sMonsterManaGaugeYOffset = BattleDisplay::MONSTER_MANA_GAUGE_Y_OFFSET;
int BattleDisplay::sHeroATBGaugeYOffset = BattleDisplay::HERO_ATB_GAUGE_Y_OFFSET;
int BattleDisplay::sMonsterATBGaugeYOffset = BattleDisplay::MONSTER_ATB_GAUGE_Y_OFFSET;
int BattleDisplay::sConditionYOffset = BattleDisplay::CONDITION_Y_OFFSET;

int BattleDisplay::sHeroHorizontalDisplayAlignVar = 0;
int BattleDisplay::sMonsterHorizontalDisplayAlignVar = 0;
int BattleDisplay::sHeroVerticalDisplayAlignVar = 0;
int BattleDisplay::sMonsterVerticalDisplayAlignVar = 0;
int BattleDisplay::sShowHeroATBGaugeVar = 0;
int BattleDisplay::sShowMonsterATBGaugeVar = 0;
int BattleDisplay::sUseSpecialATBVar = 0;
int BattleDisplay::sHeroShowConditionTurnsRemainingVar = 0;
int BattleDisplay::sMonsterShowConditionTurnsRemainingVar = 0;
int BattleDisplay::sHeroHealthGaugeDurationVar = 0;
int BattleDisplay::sMonsterHealthGaugeDurationVar = 0;
int BattleDisplay::sHeroManaGaugeDurationVar = 0;
int BattleDisplay::sMonsterManaGaugeDurationVar = 0;
int BattleDisplay::sHeroHealthBarModeVar = 0;
int BattleDisplay::sMonsterHealthBarModeVar = 0;
int BattleDisplay::sHeroManaBarModeVar = 0;
int BattleDisplay::sMonsterManaBarModeVar = 0;
int BattleDisplay::sHeroHealthChangePauseVar = 0;
int BattleDisplay::sMonsterHealthChangePauseVar = 0;
int BattleDisplay::sHeroManaChangePauseVar = 0;
int BattleDisplay::sMonsterManaChangePauseVar = 0;
int BattleDisplay::sHeroHealthChangeDurationVar = 0;
int BattleDisplay::sMonsterHealthChangeDurationVar = 0;
int BattleDisplay::sHeroManaChangeDurationVar = 0;
int BattleDisplay::sMonsterManaChangeDurationVar = 0;
int BattleDisplay::sHeroShowDisplayWhenSelectedVar = 0;
int BattleDisplay::sMonsterShowDisplayWhenSelectedVar = 0;
int BattleDisplay::sHeroShowHealthNumberVar = 0;
int BattleDisplay::sMonsterShowHealthNumberVar = 0;
int BattleDisplay::sHeroShowManaNumberVar = 0;
int BattleDisplay::sMonsterShowManaNumberVar = 0;
int BattleDisplay::sHeroConditionDurationVar = 0;
int BattleDisplay::sMonsterConditionDurationVar = 0;
int BattleDisplay::sHeroConditionBlinkDurationVar = 0;
int BattleDisplay::sMonsterConditionBlinkDurationVar = 0;
int BattleDisplay::sHeroConditionBlinkPauseVar = 0;
int BattleDisplay::sMonsterConditionBlinkPauseVar = 0;




const int NUM_HEROES = 4;                               //!< Maximum number of heroes
const int NUM_MONSTERS = 8;                             //!< Maximum number of monsters

std::map<std::string, std::string> configuration;       //!< Configuration data from the DynRPG.ini file

bool inBattle;                                          //!< Whether the game is currently in a battle

std::map<RPG::Battler *, BattleDisplay> actorBattleDisplay;     //!< Battle displays for actors
std::map<RPG::Battler *, BattleDisplay> monsterBattleDisplay;   //!< Battle displays for monsters

#ifdef DYNBATTLEDISPLAY_DEBUG
std::ofstream * ofDebug;                                //!< File output stream to record debug data
std::ofstream * BattleDisplay::ofDebug;                 //!< Pointer to same file output stream, used within BattleDisplay
#endif // DYNBATTLEDISPLAY_DEBUG

//! Called when the Actor's overlay is drawn
/*!
    OnOverlayDrawActor() is a function in Corti's BattleOverlay system which is used to signal
    that a particular Actor was drawn /after/ all Battlers have been drawn in a particular frame,
    so that UI elements can be added without them potentially being overwritten by other Battlers.
    In this plugin this function is used to call the Actor's BattleDisplay's Update() method.

    \param monPtr : ( RPG::Actor * ) Pointer to the Actor
*/
void BattleOverlay::OnOverlayDrawActor( RPG::Actor * actPtr )
{
    BattleDisplay newBattleDisplay;     // Empty BattleDisplay to use as a template for initialization

    if( 0 == actorBattleDisplay.count( actPtr ) )
    {   // Actor does not currently have a BattleDisplay
        // Create a new BattleDisplay for the Actor
        actorBattleDisplay[actPtr] = newBattleDisplay;
        actorBattleDisplay[actPtr].SetBattler( actPtr );
    }
    actorBattleDisplay[actPtr].Update();
}

//! Called when the Monster's overlay is drawn
/*!
    OnOverlayDrawMonster() is a function in Corti's BattleOverlay system which is used to signal
    that a particular Monster was drawn /after/ all Battlers have been drawn in a particular frame,
    so that UI elements can be added without them potentially being overwritten by other Battlers.
    In this plugin this function is used to call the Monster's BattleDisplay's Update() method.

    \param monPtr : ( RPG::Monster * ) Pointer to the Monster
*/
void BattleOverlay::OnOverlayDrawMonster( RPG::Monster * monPtr )
{
    BattleDisplay newBattleDisplay;     // Empty BattleDisplay to use as a template for initialization

    if( 0 == monsterBattleDisplay.count( monPtr ) )
    {   // Actor does not currently have a BattleDisplay
        // Create a new BattleDisplay for the Monster
        monsterBattleDisplay[monPtr] = newBattleDisplay;
        monsterBattleDisplay[monPtr].SetBattler( monPtr );
    }
    monsterBattleDisplay[monPtr].Update();
}

//! Called when the plugin was loaded
/*!
    onStartup() is called when the plugin is first loaded. In this plugin this method is used to
    initialize variables and load the configuration.

    \param pluginName : ( char * ) Name of the plugin
    \return (bool) false will prevent other plugins from receiving this notification, use true otherwise
*/
bool onStartup( char * pluginName )
{
    // Initialize variables
    inBattle = false;
	configuration = RPG::loadConfiguration( pluginName );
    #ifdef DYNBATTLEDISPLAY_DEBUG
    ofDebug = new std::ofstream();
    ofDebug->open( "DynPlugins\\DynBattleDisplayDebug.txt" );
    BattleDisplay::ofDebug = ofDebug;
    #endif // DYNBATTLEDISPLAY_DEBUG

	return true;
}

//! Called after the RPG objects were initialized
/*!
    onInitFinished() is called when the objects of the RPG class are finished initializing. In this
    plugin this method is used to initialize variables that are dependent on RPG objects.

    \param pluginName : ( char * ) Name of the plugin
*/
void onInitFinished()
{
    BattleDisplay newBattleDisplay;     // Empty BattleDisplay to use as a template for initialization (MUST be created AFTER BattleDisplay::InitializeStatic() was called

    // Initialize variables
    BattleDisplay::InitializeStatic( configuration );
}

//! Called before the player starts a new game
/*!
    onNewGame() is called when the player starts a new game. In this plugin this method is used to
    set any RM2K3 variables which the user specified as controller variables for this plugin's
    configuration data to the values currently held in the configuration data (which should already
    have been set by BattleDisplay::IntializeStatic() called from onInitFinished()).
*/
void onNewGame()
{
    // Call BattleDisplay's handler for initializing configuration variables according to DynRPG.ini
    BattleDisplay::InitializeConfigVariables( configuration );
    // Call BattleDisplay's handler for setting RM2K3 controller variables to configuration data
    BattleDisplay::SetControllerVariables();
}

//! Called before the player loads a game from a savestate
/*!
    onLoadGame() is called when the player loads a game from a savestate. In this plugin this method
    is used to set the BattleDisplay configuration data to the values of the RM2K3 variables
    specified for controlling configuration at run-time.

    \param 	id: (int) Savestate ID
    \param 	data: (char *) Pointer to plugin data
    \param 	length: (int) Length of plugin data
*/
void onLoadGame( int id, char * data, int length )
{
    // Call BattleDisplay's handler for setting configuration data to RM2K3 controller variables
    BattleDisplay::ConfigUpdate();
}

//! Called every frame
/*!
    onFrame() is called on every frame of the game loop. In this plugin this method is used to
    detect and react to transitions between battles and other scenes.

    \param scene : ( RPG::Scene ) Enumerator for what type of scene the game is currently in
*/
void onFrame( RPG::Scene scene )
{
    if( inBattle )
    {   // Game was in a battle scene at last check
        if( RPG::SCENE_BATTLE != scene )
        {   // Current scene is not a battle; battle just ended!
            // Clear BattleDisplay maps
            actorBattleDisplay.clear();
            monsterBattleDisplay.clear();
            inBattle = false;
        }
    }
    else if( RPG::SCENE_BATTLE == scene )
    {   // Current scene is a battle; battle just started!
        inBattle = true;
    }
}

//! Called immediately after a Battler is drawn
/*!
    onBattlerDrawn() is called immediately after a Battler is drawn to the Canvas. In this plugin
    this method is used to call the RememberBattlerDrawn() function of Corti's BattleOverlay system.

    \param battler : ( RPG::Battler * ) The battler which was drawn (or supposed to be drawn)
    \param isMonster: ( bool ) true if the battler is a monster
    \param id: ( int ) Zero-based party member ID of the battler
    \return ( bool ) false will prevent other plugins from receiving this notification, use true otherwise
*/
bool onBattlerDrawn( RPG::Battler *battler, bool isMonster, int id )
{
    BattleOverlay::RememberBattlerDrawn( battler );
    return true;
}

//! Called before the battle status window is drawn
/*!
    onDrawBattleActionWindow() is called before the battle status window is drawn. In this plugin
    this method is used to call the DrawOverlay() function of Corti's BattleOverlay system.

    \param x : ( int * ) Pointer to the X coordinate of the upper-left corner of the window
    \param y: ( int * ) Pointer to the Y coordinate of the upper-left corner of the window
    \param selection: ( int ) Zero-based index of the current selection
    \param selActive: ( bool ) true if the selection parameter contains a valid value and a choice bar is drawn
    \param isVisible: ( bool ) true if the window is visible
    \return ( bool ) false will prevent the original window from being drawn and other plugins from receiving this notification, use true otherwise
*/
bool onDrawBattleActionWindow( int * x, int * y, int selection, bool selActive, bool isVisible )
{
    // Call DrawOverlay iff selActive is false
    if( !selActive )
    {
        BattleOverlay::DrawOverlay();
    }
    return true;
}

//! Respond to potential comment commands
/*!
    onComment() is called when the game runs across a comment line in its event scripting. Comment
    commands are how the user communicates with the plugin, so this function checks for comment
    commands and responds accordingly.

    \param text (const char*) The comment's content as simple text
    \param parsedData (const RPG::ParsedCommentData*) The already parsed data
    \param nextScriptLine (RPG::EventScriptLine*) The next event script line after the comment
    \param scriptData (RPG::EventScriptData*) Pointer to the RPG::EventScriptData object of the current event script
    \param eventId (int) The ID of the current event (negative for common events, zero for battle events)
    \param pageId (int) The ID of the current event page (zero for common and battle events - sorry, no battle event page ID yet)
    \param lineId (int) The zero-based line number
    \param nextLineId (int*) Pointer to the next executed line number (-1 for default)
    \return (bool) false will prevent other plugins from receiving this notification, use true otherwise
*/
bool onComment( const char* text,
                const RPG::ParsedCommentData* 	parsedData,
                RPG::EventScriptLine* 	nextScriptLine,
                RPG::EventScriptData* 	scriptData,
                int 	eventId,
                int 	pageId,
                int 	lineId,
                int* 	nextLineId )
{
    static std::string cmd;

    // Check for and respond to plugin comment commands
    cmd = parsedData->command;
    if( 0 == cmd.compare( "dynbattledisplay_update" ) )
    {   // Update the configuration data according to values held in reserved RM2K3 variables
        BattleDisplay::ConfigUpdate();
        return false;
    }
    if( 0 == cmd.compare( "dynbattledisplay_default_config" ) )
    {   // Reset the configuration data and the associated RM2K3 variables to the values provided in
        // DynRPG.ini
        BattleDisplay::InitializeConfigVariables( configuration );
        BattleDisplay::SetControllerVariables();
        return false;
    }
    // No comment commands for this plugin detected; pass notification on for other plugins
    return true;
}

//! Clean up after use
/*!
    onExit() is called when the game closes. In this plugin this is used to perform any needed
    cleanup.
*/
void onExit()
{
    int i;          // Index variable
    std::map<int, RPG::Image *>::iterator iter;     // Iterator for traversing a map of Image pointers

    // Destroy static Images of BattleDisplay class
    RPG::Image::destroy( BattleDisplay::sHealthGaugePtr );
    RPG::Image::destroy( BattleDisplay::sManaGaugePtr );
    RPG::Image::destroy( BattleDisplay::sATBGaugePtr );
    RPG::Image::destroy( BattleDisplay::sHealthBarAPtr );
    RPG::Image::destroy( BattleDisplay::sHealthBarBPtr );
    RPG::Image::destroy( BattleDisplay::sManaBarAPtr );
    RPG::Image::destroy( BattleDisplay::sManaBarBPtr );
    RPG::Image::destroy( BattleDisplay::sATBBarAPtr );
    RPG::Image::destroy( BattleDisplay::sATBBarBPtr );
    RPG::Image::destroy( BattleDisplay::sFastATBBarAPtr );
    RPG::Image::destroy( BattleDisplay::sFastATBBarBPtr );
    RPG::Image::destroy( BattleDisplay::sSlowATBBarAPtr );
    RPG::Image::destroy( BattleDisplay::sSlowATBBarBPtr );
    RPG::Image::destroy( BattleDisplay::sSystemDisplayPtr );
    if( BattleDisplay::sIconDisplayPtr )
    {
        RPG::Image::destroy( BattleDisplay::sIconDisplayPtr );
    }
    for( i = 0; i < BattleDisplay::NUM_DIGITS; i++ )
    {
        RPG::Image::destroy( BattleDisplay::sDigitPtr[i] );
    }
    for( iter = BattleDisplay::sConditionIconPtr.begin(); iter != BattleDisplay::sConditionIconPtr.end(); ++iter )
    {
        RPG::Image::destroy( iter->second );
    }
    #ifdef DYNBATTLEDISPLAY_DEBUG
    ofDebug->close();
    #endif // DYNBATTLEDISPLAY_DEBUG
}
